@@ -764,7 +764,7 @@ printf("file_list_level: %i\n", file_list_level);
 		}
 
 		if ( ftsent->fts_pathlen < sub_index ) {
-			printf("ERROR: fts_pathlen error at %s line %d\n", __FILE__, __LINE__);
+			LogError("ERROR: fts_pathlen error at %s line %d\n", __FILE__, __LINE__);
 			exit(250);
 		}
 		fts_path = &ftsent->fts_path[sub_index];
@@ -798,6 +798,9 @@ printf("DGB: short fts: '%s', filer_first: '%s', filter_last: '%s'\n",
 					continue;
 				if ( strstr(ftsent->fts_name, ".stat") != NULL )
 					continue;
+				// skip OSX DS_Store files
+				if ( strstr(ftsent->fts_name, ".DS_Store") != NULL )
+					continue;
 				// skip pcap file
 				if ( strstr(ftsent->fts_name, "pcap") != NULL )
 					continue;
