@@ -92,6 +92,7 @@ typedef struct exporter_v5_s {
 	uint64_t	packets;			// number of packets sent by this exporter
 	uint64_t	flows;				// number of flow records sent by this exporter
 	uint32_t	sequence_failure;	// number of sequence failues
+	uint32_t	padding_errors;		// number of sequence failues
 
 	// generic sampler
 	generic_sampler_t		*sampler;
@@ -212,6 +213,7 @@ char ipstr[IP_STRING_LEN];
 	(*e)->info.ip			= fs->ip;
 	(*e)->info.sa_family	= fs->sa_family;
 	(*e)->sequence_failure	= 0;
+	(*e)->padding_errors	= 0;
 	(*e)->packets			= 0;
 	(*e)->flows				= 0;
 	(*e)->first	 			= 1;
@@ -721,10 +723,10 @@ uint32_t	i, id, t1, t2;
   	v5_output_record->dPkts		= htonl((uint32_t)master_record->dPkts);
   	v5_output_record->dOctets	= htonl((uint32_t)master_record->dOctets);
 
-  	v5_output_record->input		= 0;
-  	v5_output_record->output	= 0;
-  	v5_output_record->src_as	= 0;
-  	v5_output_record->dst_as	= 0;
+  	v5_output_record->input		= htons(master_record->input);
+  	v5_output_record->output	= htons(master_record->output);
+  	v5_output_record->src_as	= htons(master_record->srcas);
+  	v5_output_record->dst_as	= htons(master_record->dstas);
 	v5_output_record->src_mask 	= 0;
 	v5_output_record->dst_mask 	= 0;
 	v5_output_record->pad1 		= 0;
