@@ -1,8 +1,5 @@
 /*
- *  Copyright (c) 2017, Peter Haag
- *  Copyright (c) 2016, Peter Haag
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2018, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -323,7 +320,7 @@ static struct format_token_list_s {
 	{ "%ibyt", 0, " In Byte", 			 	String_InBytes },		// In Bytes
 	{ "%obyt", 0, "Out Byte", 			 	String_OutBytes },		// In Bytes
 	{ "%fl",  0, "Flows", 				 	String_Flows },			// Flows
-	{ "%flg", 0,  " Flags", 			 	String_Flags },			// TCP Flags
+	{ "%flg", 0,  "   Flags", 			 	String_Flags },			// TCP Flags
 	{ "%tos", 0, "Tos", 				 	String_Tos },			// Tos - compat
 	{ "%stos", 0, "STos", 				 	String_SrcTos },		// Tos - Src tos
 	{ "%dtos", 0, "DTos", 				 	String_DstTos },		// Tos - Dst tos
@@ -493,7 +490,7 @@ char protolist[NumProtos][MAX_PROTO_STR] = {
 	"TTP  ",    // 84	TTP
 	"NSIGP",    // 85	NSFNET-IGP
 	"DGP  ",    // 86	Dissimilar Gateway Protocol
-	"TCP  ",    // 87	TCF
+	"TCF  ",    // 87	TCF
 	"EIGRP",    // 88	EIGRP
 	"OSPF ",    // 89	OSPFIGP
 	"S-RPC",    // 90	Sprite RPC Protocol
@@ -658,7 +655,7 @@ char *Get_fwd_status_name(uint32_t id) {
 
 } // End of Get_fwd_status_name
 
-void format_file_block_header(void *header, char ** s, int tag) {
+void format_file_block_header(void *header, char **s, int tag) {
 data_block_header_t *h = (data_block_header_t *)header;
 	
 	snprintf(data_string,STRINGSIZE-1 ,""
@@ -673,7 +670,7 @@ data_block_header_t *h = (data_block_header_t *)header;
 
 } // End of format_file_block_header
 
-void format_file_block_record(void *record, char ** s, int tag) {
+void format_file_block_record(void *record, char **s, int tag) {
 char 		*_s, as[IP_STRING_LEN], ds[IP_STRING_LEN], datestr1[64], datestr2[64], datestr3[64], flags_str[16];
 char		s_snet[IP_STRING_LEN], s_dnet[IP_STRING_LEN], s_proto[32];
 int			i, id;
@@ -874,29 +871,23 @@ extension_map_t	*extension_map = r->map_ref;
 				slen = STRINGSIZE - _slen;
 				break;
 			case EX_NEXT_HOP_v4:
-				as[0] = 0;
-				r->ip_nexthop.V4 = htonl(r->ip_nexthop.V4);
-				inet_ntop(AF_INET, &r->ip_nexthop.V4, as, sizeof(as));
-				as[IP_STRING_LEN-1] = 0;
-
-				snprintf(_s, slen-1,
-"  ip next hop  =  %16s\n"
-, as);
-				_slen = strlen(data_string);
-				_s = data_string + _slen;
-				slen = STRINGSIZE - _slen;
-	
-			break;
 			case EX_NEXT_HOP_v6:
-				as[0] = 0;
-				r->ip_nexthop.V6[0] = htonll(r->ip_nexthop.V6[0]);
-				r->ip_nexthop.V6[1] = htonll(r->ip_nexthop.V6[1]);
-				inet_ntop(AF_INET6, r->ip_nexthop.V6, as, sizeof(as));
-				if ( ! long_v6 ) {
-					condense_v6(as);
-					condense_v6(ds);
+				if ( (r->flags & FLAG_IPV6_NH ) != 0 ) { // IPv6
+					as[0] = 0;
+					r->ip_nexthop.V6[0] = htonll(r->ip_nexthop.V6[0]);
+					r->ip_nexthop.V6[1] = htonll(r->ip_nexthop.V6[1]);
+					inet_ntop(AF_INET6, r->ip_nexthop.V6, as, sizeof(as));
+					if ( ! long_v6 ) {
+						condense_v6(as);
+						condense_v6(ds);
+					}
+					as[IP_STRING_LEN-1] = 0;
+				} else {
+					as[0] = 0;
+					r->ip_nexthop.V4 = htonl(r->ip_nexthop.V4);
+					inet_ntop(AF_INET, &r->ip_nexthop.V4, as, sizeof(as));
+					as[IP_STRING_LEN-1] = 0;
 				}
-				as[IP_STRING_LEN-1] = 0;
 
 				snprintf(_s, slen-1,
 "  ip next hop  =  %16s\n"
@@ -906,30 +897,23 @@ extension_map_t	*extension_map = r->map_ref;
 				slen = STRINGSIZE - _slen;
 			break;
 			case EX_NEXT_HOP_BGP_v4:
-				as[0] = 0;
-				r->bgp_nexthop.V4 = htonl(r->bgp_nexthop.V4);
-				inet_ntop(AF_INET, &r->bgp_nexthop.V4, as, sizeof(as));
-				as[IP_STRING_LEN-1] = 0;
-
-				snprintf(_s, slen-1,
-"  bgp next hop =  %16s\n"
-, as);
-				_slen = strlen(data_string);
-				_s = data_string + _slen;
-				slen = STRINGSIZE - _slen;
-	
-			break;
 			case EX_NEXT_HOP_BGP_v6:
-				as[0] = 0;
-				r->bgp_nexthop.V6[0] = htonll(r->bgp_nexthop.V6[0]);
-				r->bgp_nexthop.V6[1] = htonll(r->bgp_nexthop.V6[1]);
-				inet_ntop(AF_INET6, r->ip_nexthop.V6, as, sizeof(as));
-				if ( ! long_v6 ) {
-					condense_v6(as);
-					condense_v6(ds);
+				if ( (r->flags & FLAG_IPV6_NHB ) != 0 ) { // IPv6
+					as[0] = 0;
+					r->bgp_nexthop.V6[0] = htonll(r->bgp_nexthop.V6[0]);
+					r->bgp_nexthop.V6[1] = htonll(r->bgp_nexthop.V6[1]);
+					inet_ntop(AF_INET6, r->ip_nexthop.V6, as, sizeof(as));
+					if ( ! long_v6 ) {
+						condense_v6(as);
+						condense_v6(ds);
+					}
+					as[IP_STRING_LEN-1] = 0;
+				} else {
+					as[0] = 0;
+					r->bgp_nexthop.V4 = htonl(r->bgp_nexthop.V4);
+					inet_ntop(AF_INET, &r->bgp_nexthop.V4, as, sizeof(as));
+					as[IP_STRING_LEN-1] = 0;
 				}
-				as[IP_STRING_LEN-1] = 0;
-
 				snprintf(_s, slen-1,
 "  bgp next hop =  %16s\n"
 , as);
@@ -1416,7 +1400,7 @@ master_record_t *r = (master_record_t *)record;
 		slen = STRINGSIZE - _slen;
 	}
 	
-	if ( (r->flags & FLAG_IPV6_NH ) != 0 ) { // IPv6
+	if ( (r->flags & FLAG_IPV6_NHB ) != 0 ) { // IPv6
 		// EX_NEXT_HOP_BGP_v6:
 		as[0] = 0;
 		r->bgp_nexthop.V6[0] = htonll(r->bgp_nexthop.V6[0]);
@@ -2052,7 +2036,7 @@ static void String_BGPNextHop(master_record_t *r, char *string) {
 char tmp_str[IP_STRING_LEN];
 
 	tmp_str[0] = 0;
-	if ( (r->flags & FLAG_IPV6_NH ) != 0 ) { // IPv6
+	if ( (r->flags & FLAG_IPV6_NHB ) != 0 ) { // IPv6
 		uint64_t	ip[2];
 
 		ip[0] = htonll(r->bgp_nexthop.V6[0]);
@@ -2387,18 +2371,15 @@ static void String_FwdStatus(master_record_t *r, char *string) {
 
 static void String_Flags(master_record_t *r, char *string) {
 
-	// if record contains unusuall flags, print the flags in hex as 0x.. number
-	if ( r->tcp_flags > 63 ) {
-		snprintf(string, 7, "  0x%2x", r->tcp_flags );
-	} else {
-		string[0] = r->tcp_flags & 32 ? 'U' : '.';
-		string[1] = r->tcp_flags & 16 ? 'A' : '.';
-		string[2] = r->tcp_flags &  8 ? 'P' : '.';
-		string[3] = r->tcp_flags &  4 ? 'R' : '.';
-		string[4] = r->tcp_flags &  2 ? 'S' : '.';
-		string[5] = r->tcp_flags &  1 ? 'F' : '.';
-	}
-	string[6] = '\0';
+	string[0] = r->tcp_flags & 128 ? 'C' : '.';	// Congestion window reduced -  CWR
+	string[1] = r->tcp_flags &  64 ? 'E' : '.';	// ECN-Echo
+	string[2] = r->tcp_flags &  32 ? 'U' : '.';	// Urgent
+	string[3] = r->tcp_flags &  16 ? 'A' : '.';	// Ack
+	string[4] = r->tcp_flags &   8 ? 'P' : '.';	// Push
+	string[5] = r->tcp_flags &   4 ? 'R' : '.';	// Reset
+	string[6] = r->tcp_flags &   2 ? 'S' : '.';	// Syn
+	string[7] = r->tcp_flags &   1 ? 'F' : '.';	// Fin
+	string[8] = '\0';
 
 } // End of String_Flags
 
