@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2018, 2017 Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2019, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -90,10 +89,6 @@ FilterEngine_data_t	*Engine;
 extern char	*FilterFilename;
 extern uint32_t loopcnt;
 
-#ifdef COMPAT15
-extern extension_descriptor_t extension_descriptor[];
-#endif
-
 /* Local Variables */
 const char *nfdump_version = VERSION;
 
@@ -306,7 +301,7 @@ static void usage(char *name) {
 					"\t\t json     json output format.\n"
 					"\t\t pipe     '|' separated legacy machine parseable output format.\n"
 					"\t\t\tmode may be extended by '6' for full IPv6 listing. e.g.long6, extended6.\n"
-					"-E <file>\tPrint exporter ans sampling info for collected flows.\n"
+					"-E <file>\tPrint exporter and sampling info for collected flows.\n"
 					"-v <file>\tverify netflow data file. Print version and blocks.\n"
 					"-x <file>\tverify extension records in netflow data file.\n"
 					"-X\t\tDump Filtertable and exit (debug option).\n"
@@ -348,11 +343,11 @@ char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_
 			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
 			(long long unsigned)pps, (long long unsigned)bpp );
 	} else {
-		format_number(stat_record->numbytes, byte_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(stat_record->numpackets, packet_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(bps, bps_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(pps, pps_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(bpp, bpp_str, DONT_SCALE_NUMBER, VAR_LENGTH);
+		format_number(stat_record->numbytes, byte_str, DO_SCALE_NUMBER, VAR_LENGTH);
+		format_number(stat_record->numpackets, packet_str, DO_SCALE_NUMBER, VAR_LENGTH);
+		format_number(bps, bps_str, DO_SCALE_NUMBER, VAR_LENGTH);
+		format_number(pps, pps_str, DO_SCALE_NUMBER, VAR_LENGTH);
+		format_number(bpp, bpp_str, DO_SCALE_NUMBER, VAR_LENGTH);
 		printf("Summary: total flows: %llu, total bytes: %s, total packets: %s, avg bps: %s, avg pps: %s, avg bpp: %s\n",
 		(unsigned long long)stat_record->numflows, byte_str, packet_str, bps_str, pps_str, bpp_str );
 	}
@@ -368,10 +363,6 @@ nffile_t			*nffile_w, *nffile_r;
 stat_record_t 		stat_record;
 int 				done, write_file;
 
-#ifdef COMPAT15
-int	v1_map_done = 0;
-#endif
-	
 	// time window of all matched flows
 	memset((void *)&stat_record, 0, sizeof(stat_record_t));
 	stat_record.first_seen = 0x7fffffff;
@@ -431,8 +422,6 @@ int	v1_map_done = 0;
 	done = 0;
 	while ( !done ) {
 	int i, ret;
-	char *ConvertBuffer = NULL;
-
 		// get next data block from file
 		ret = ReadBlock(nffile_r);
 
@@ -467,52 +456,6 @@ int	v1_map_done = 0;
 				total_bytes += ret;
 		}
 
-
-#ifdef COMPAT15
-		if ( nffile_r->block_header->id == DATA_BLOCK_TYPE_1 ) {
-			common_record_v1_t *v1_record = (common_record_v1_t *)nffile_r->buff_ptr;
-			// create an extension map for v1 blocks
-			if ( v1_map_done == 0 ) {
-				extension_map_t *map = malloc(sizeof(extension_map_t) + 2 * sizeof(uint16_t) );
-				if ( ! map ) {
-					LogError("malloc() allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-					exit(255);
-				}
-				map->type 	= ExtensionMapType;
-				map->size 	= sizeof(extension_map_t) + 2 * sizeof(uint16_t);
-				if (( map->size & 0x3 ) != 0 ) {
-					map->size += 4 - ( map->size & 0x3 );
-				}
-
-				map->map_id = INIT_ID;
-
-				map->ex_id[0]  = EX_IO_SNMP_2;
-				map->ex_id[1]  = EX_AS_2;
-				map->ex_id[2]  = 0;
-				
-				map->extension_size  = 0;
-				map->extension_size += extension_descriptor[EX_IO_SNMP_2].size;
-				map->extension_size += extension_descriptor[EX_AS_2].size;
-
-				if ( Insert_Extension_Map(extension_map_list,map) && write_file ) {
-					// flush new map
-					AppendToBuffer(nffile_w, (void *)map, map->size);
-				} // else map already known and flushed
-
-				v1_map_done = 1;
-			}
-
-			// convert the records to v2
-			for ( i=0; i < nffile_r->block_header->NumRecords; i++ ) {
-				common_record_t *v2_record = (common_record_t *)v1_record;
-				Convert_v1_to_v2((void *)v1_record);
-				// now we have a v2 record -> use size of v2_record->size
-				v1_record = (common_record_v1_t *)((pointer_addr_t)v1_record + v2_record->size);
-			}
-			nffile_r->block_header->id = DATA_BLOCK_TYPE_2;
-		}
-#endif
-
 		if ( nffile_r->block_header->id == Large_BLOCK_Type ) {
 			// skip
 			printf("Xstat block skipped ...\n");
@@ -529,26 +472,19 @@ int	v1_map_done = 0;
 			continue;
 		}
 
+		uint32_t sumSize = 0;
 		record_ptr = nffile_r->buff_ptr;
 		for ( i=0; i < nffile_r->block_header->NumRecords; i++ ) {
 			flow_record = record_ptr;
+			if ( (sumSize + record_ptr->size) > ret || (record_ptr->size < sizeof(record_header_t)) ) {
+				LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
+				exit(255);
+			}
+			sumSize += record_ptr->size;
 			switch ( record_ptr->type ) {
 				case CommonRecordV0Type: 
-					// convert common record v0
-					if ( !ConvertBuffer ) {
-						ConvertBuffer = malloc(65536);
-						if ( !ConvertBuffer ) {
-							LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-							exit(255);
-						}
-					}
-					if ( !ConvertCommonV0((void *)record_ptr, (common_record_t *)ConvertBuffer) ) {
-						LogError("Corrupt data file. Unable to decode at %s line %d\n", __FILE__, __LINE__);
-						exit(255);
-
-					}
-					flow_record = (common_record_t *)ConvertBuffer;
-					dbg_printf("Converted type %u to %u record\n", CommonRecordV0Type, CommonRecordType);
+					LogError("Old common v0 records no longer supported - skipped");
+					break;
 				case CommonRecordType: {
 					int match;
 					uint32_t map_id;
@@ -635,10 +571,18 @@ int	v1_map_done = 0;
 				case ExtensionMapType: {
 					extension_map_t *map = (extension_map_t *)record_ptr;
 	
-					if ( Insert_Extension_Map(extension_map_list, map) && write_file ) {
-						// flush new map
-						AppendToBuffer(nffile_w, (void *)map, map->size);
-					} // else map already known and flushed
+					int ret = Insert_Extension_Map(extension_map_list, map);
+					switch (ret) {
+						case 0:
+							break; // map already known and flushed
+						case 1:
+							if ( write_file ) 
+								AppendToBuffer(nffile_w, (void *)map, map->size);
+							break;
+						default:
+							LogError("Corrupt data file. Unable to decode at %s line %d\n", __FILE__, __LINE__);
+							exit(255);
+					}
 					} break;
 				case ExporterRecordType:
 				case SamplerRecordype:
