@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,17 +29,12 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nflowcache.c 40 2009-12-16 10:41:44Z haag $
- *
- *  $LastChangedRevision: 40 $
- *	
  */
 
 #include "config.h"
 
 #include <stdio.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -53,6 +50,7 @@
 #endif
 
 #include "nffile.h"
+#include "nfx.h"
 #include "nflowcache.h"
 
 #ifndef DEVEL
@@ -113,6 +111,14 @@ static struct aggregate_info_s {
 	{ "srcip",		{ 8, OffsetSrcIPv6b, 	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
 	{ "dstip",		{ 8, OffsetDstIPv6a, 	MaskIPv6, 	 ShiftIPv6 },    	-1, 0,	"%da"	},
 	{ "dstip",		{ 8, OffsetDstIPv6b, 	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
+#ifdef NSEL
+	{ "xsrcip",		{ 8, OffsetXLATESRCv6a,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0,	"%xsa" 	},
+	{ "xsrcip",		{ 8, OffsetXLATESRCv6b,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
+	{ "xdstip",		{ 8, OffsetXLATEDSTv6a,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0,	"%xda"	},
+	{ "xdstip",		{ 8, OffsetXLATESRCv6b,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
+	{ "xsrcport",	{ 2, OffsetXLATEPort,   MaskXLATESRCPORT,   ShiftXLATESRCPORT }, 	-1, 0, 	"%xsp"	},
+	{ "xdstport",	{ 2, OffsetXLATEPort,   MaskXLATEDSTPORT,   ShiftXLATEDSTPORT }, 	-1, 0, 	"%xdp"	},
+#endif
 	{ "dstip4",		{ 8, OffsetDstIPv6a, 	MaskIPv6, 	 ShiftIPv6 },     	 0, 0,	"%da"	},
 	{ "dstip4",		{ 8, OffsetDstIPv6b, 	MaskIPv6, 	 ShiftIPv6 },     	 1, 0,	NULL	},
 	{ "dstip6",		{ 8, OffsetDstIPv6a, 	MaskIPv6, 	 ShiftIPv6 },     	 0, 0,	"%da"	},
@@ -404,7 +410,7 @@ uint32_t index = index_cache & FlowTable.IndexMask;
 
 } // End of hash_insert_FlowTable
 
-void InsertFlow(common_record_t *raw_record, master_record_t *flow_record) {
+void InsertFlow(common_record_t *raw_record, master_record_t *flow_record, extension_info_t *extension_info) {
 FlowTableRecord_t	*record;
 
 	// allocate enough memory for the new flow including all additional information in FlowTableRecord_t
@@ -424,7 +430,7 @@ FlowTableRecord_t	*record;
 	FlowTable.bucketcache[0] = record;
 	
 	// safe the extension map and exporter reference
-	record->map_ref = flow_record->map_ref;
+	record->map_info_ref = extension_info;
 	record->exp_ref = flow_record->exp_ref;
 
 	record->counter[INBYTES]	 = flow_record->dOctets;
@@ -438,7 +444,7 @@ FlowTableRecord_t	*record;
 
 
 
-void AddFlow(common_record_t *raw_record, master_record_t *flow_record ) {
+void AddFlow(common_record_t *raw_record, master_record_t *flow_record, extension_info_t *extension_info ) {
 static void			*keymem = NULL, *bidirkeymem = NULL;
 FlowTableRecord_t	*FlowTableRecord;
 uint32_t			index_cache; 
@@ -475,8 +481,8 @@ uint32_t			index_cache;
 			FlowTableRecord->flowrecord.msec_last = flow_record->msec_last;
 		}
 
-		FlowTableRecord->counter[FLOWS]++;
-		FlowTableRecord->flowrecord.tcp_flags		  |= flow_record->tcp_flags;
+		FlowTableRecord->counter[FLOWS]        += flow_record->aggr_flows ? flow_record->aggr_flows : 1;
+		FlowTableRecord->flowrecord.tcp_flags  |= flow_record->tcp_flags;
 
 	} else if ( !bidir_flows || ( flow_record->prot != IPPROTO_TCP && flow_record->prot != IPPROTO_UDP) ) {
 		// no flow record found and no TCP/UDP bidir flows. Insert flow record into hash
@@ -488,7 +494,7 @@ uint32_t			index_cache;
 		FlowTableRecord->counter[OUTPACKETS] = flow_record->out_pkts;
 		FlowTableRecord->counter[FLOWS]   	 = flow_record->aggr_flows ? flow_record->aggr_flows : 1;
 
-		FlowTableRecord->map_ref  	 		 = flow_record->map_ref;
+		FlowTableRecord->map_info_ref  	 	 = extension_info;
 		FlowTableRecord->exp_ref  	 		 = flow_record->exp_ref;
 
 		// keymen got part of the cache
@@ -529,8 +535,8 @@ uint32_t			index_cache;
 				FlowTableRecord->flowrecord.msec_last = flow_record->msec_last;
 			}
 	
-			FlowTableRecord->counter[FLOWS]++;
-			FlowTableRecord->flowrecord.tcp_flags		  |= flow_record->tcp_flags;
+			FlowTableRecord->counter[FLOWS]        += flow_record->aggr_flows ? flow_record->aggr_flows : 1;
+			FlowTableRecord->flowrecord.tcp_flags  |= flow_record->tcp_flags;
 		} else {
 			// no bidir flow found 
 			// insert original flow into the cache
@@ -541,7 +547,7 @@ uint32_t			index_cache;
 			FlowTableRecord->counter[OUTBYTES]   = flow_record->out_bytes;
 			FlowTableRecord->counter[OUTPACKETS] = flow_record->out_pkts;
 			FlowTableRecord->counter[FLOWS]   	 = flow_record->aggr_flows ? flow_record->aggr_flows : 1;
-			FlowTableRecord->map_ref  	 		 = flow_record->map_ref;
+			FlowTableRecord->map_info_ref  	 	 = extension_info;
 			FlowTableRecord->exp_ref  	 		 = flow_record->exp_ref;
 
 			keymem = NULL;
@@ -636,6 +642,7 @@ struct aggregate_info_s *a;
 
 	stack_count = 0;
 	subnet 		= 0;
+	has_mask    = 0;
 
 	aggregate_key_len = 0;
 
@@ -907,20 +914,20 @@ Default_key_t *keyptr;
 	} else if ( swap_flow ) {
 		// default 5-tuple aggregation for bidirectional flows
 		keyptr = (Default_key_t *)keymem;
-		keyptr->srcaddr[0]	= flow_record->v6.dstaddr[0];
-		keyptr->srcaddr[1]	= flow_record->v6.dstaddr[1];
-		keyptr->dstaddr[0]	= flow_record->v6.srcaddr[0];
-		keyptr->dstaddr[1]	= flow_record->v6.srcaddr[1];
+		keyptr->srcaddr[0]	= flow_record->V6.dstaddr[0];
+		keyptr->srcaddr[1]	= flow_record->V6.dstaddr[1];
+		keyptr->dstaddr[0]	= flow_record->V6.srcaddr[0];
+		keyptr->dstaddr[1]	= flow_record->V6.srcaddr[1];
 		keyptr->srcport		= flow_record->dstport;
 		keyptr->dstport		= flow_record->srcport;
 		keyptr->proto		= flow_record->prot;
 	} else {
 		// default 5-tuple aggregation
 		keyptr = (Default_key_t *)keymem;
-		keyptr->srcaddr[0]	= flow_record->v6.srcaddr[0];
-		keyptr->srcaddr[1]	= flow_record->v6.srcaddr[1];
-		keyptr->dstaddr[0]	= flow_record->v6.dstaddr[0];
-		keyptr->dstaddr[1]	= flow_record->v6.dstaddr[1];
+		keyptr->srcaddr[0]	= flow_record->V6.srcaddr[0];
+		keyptr->srcaddr[1]	= flow_record->V6.srcaddr[1];
+		keyptr->dstaddr[0]	= flow_record->V6.dstaddr[0];
+		keyptr->dstaddr[1]	= flow_record->V6.dstaddr[1];
 		keyptr->srcport		= flow_record->srcport;
 		keyptr->dstport		= flow_record->dstport;
 		keyptr->proto		= flow_record->prot;
