@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2016, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,15 +29,11 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nfstatfile.c 39 2009-11-25 08:11:15Z haag $
- *
- *  $LastChangedRevision: 39 $
- *  
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,6 +52,7 @@
 #include <stdint.h>
 #endif
 
+#include "util.h"
 #include "nfstatfile.h"
 
 #define stat_filename ".nfstat"
@@ -94,13 +93,6 @@ static const double _1hour = 3600.0;
 static const double _1day  = 86400.0;
 static const double _1week = 604800.0;
 
-/* 
- * expire.c is needed for daemon code as well as normal stdio code 
- * therefore a generic LogError is defined, which maps to the 
- * approriate logging channel - either stderr or syslog
- */
-void LogError(char *format, ...);
-
 static inline uint64_t string2uint64(char *s);
 
 static int ParseString(char *str, char **key, char **value);
@@ -233,7 +225,7 @@ static void VerifyStatInfo(dirstat_t *statinfo) {
  *	dirname: 	directory to read the .nfstat file
  *	dirstat_p:	Assign a point of the result to this pointer
  *	lock:		READ_ONLY file is locked while reading, and unlocked and closed thereafter
- *				CRETAE_AND_LOCK if file does not exists, create it - continue as LOCK_IF_EXISTS
+ *				CREATE_AND_LOCK if file does not exists, create it - continue as LOCK_IF_EXISTS
  *				LOCK_IF_EXISTS: lock the file if it exists - file remains open
  * If file does not exists, an empty record is returned.
  */
@@ -241,9 +233,8 @@ int ReadStatInfo(char *dirname, dirstat_t **dirstat_p, int lock ) {
 struct stat filestat;
 char *in_buff, *s, *p, *k, *v;
 char filename[MAXPATHLEN];
-int fd, err, r_size, new_created, next_free;
+int fd, err, r_size, next_free;
 
-	new_created = 0;
 	*dirstat_p = NULL;
 
 	// if the dirstack does not exist, creat it
