@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,12 +29,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nftest.c 39 2009-11-25 08:11:15Z haag $
- *
- *  $LastChangedRevision: 39 $
- *	
  */
 
 #include "config.h"
@@ -74,37 +70,22 @@
 #include "nftree.h"
 #include "nffile.h"
 #include "nf_common.h"
+#include "nfx.h"
 #include "util.h"
 
 /* Global Variables */
 extern char 	*CurrentIdent;
+extern extension_descriptor_t extension_descriptor[];
 
 FilterEngine_data_t	*Engine;
 
 /* exported fuctions */
-void LogError(char *format, ...);
-
 int check_filter_block(char *filter, master_record_t *flow_record, int expect);
 
 void check_offset(char *text, pointer_addr_t offset, pointer_addr_t expect);
 
 void CheckCompression(char *filename);
 
-/* 
- * some modules are needed for daemon code as well as normal stdio code 
- * therefore a generic LogError is defined, which maps in this case
- * to stderr
- */
-void LogError(char *format, ...) {
-va_list var_args;
-
-	va_start(var_args, format);
-	vfprintf(stderr, format, var_args);
-	va_end(var_args);
-
-} // End of LogError
-
-
 int check_filter_block(char *filter, master_record_t *flow_record, int expect) {
 int ret, i;
 uint64_t	*block = (uint64_t *)flow_record;
@@ -151,10 +132,11 @@ nffile_t	*nffile_w, *nffile_r;
 int i, compress, bsize;
 ssize_t	ret;
 char outfile[MAXPATHLEN];
-struct timeval  	tstart[2];
-struct timeval  	tend[2];
+struct timeval  	tstart[4];
+struct timeval  	tend[4];
 u_long usec, sec;
-double wall[2];
+double wall[4];
+uint32_t recsize[4];
 
 	nffile_r = OpenFile(filename, NULL);
 	if ( !nffile_r ) {
@@ -175,17 +157,20 @@ double wall[2];
 	nffile_w = NULL;
 
 	bsize = nffile_r->block_header->size;
-	for ( compress=0; compress<=1; compress++ ) {
+	for ( compress=NOT_COMPRESSED; compress<=LZ4_COMPRESSED; compress++ ) {
+		int wsize;
 		nffile_w = OpenNewFile(outfile, nffile_w, compress, 0, NULL);
 		if ( !nffile_w ) {
 			DisposeFile(nffile_r);
         	return;
     	}
-		
+		// fill buffer
+		memcpy(nffile_w->buff_pool[0], nffile_r->buff_pool[0], nffile_r->buff_size);
 		gettimeofday(&(tstart[compress]), (struct timezone*)NULL);
 		for ( i=0; i<100; i++ ) {
 			nffile_w->block_header->size = bsize;
-			if ( WriteExtraBlock(nffile_w, nffile_r->block_header) <= 0 ) {
+			wsize = WriteBlock(nffile_w);
+			if ( wsize <= 0 ) {
 				fprintf(stderr, "Failed to write output buffer to disk: '%s'" , strerror(errno));
 				// Cleanup
 				CloseFile(nffile_w);
@@ -208,21 +193,18 @@ double wall[2];
 		sec  = tend[compress].tv_sec - tstart[compress].tv_sec;
 
 		wall[compress] = (double)sec + ((double)usec)/1000000;
+		recsize[compress] = wsize;
 	}
 
 	DisposeFile(nffile_r);
 	DisposeFile(nffile_w);
 
 	printf("100 write cycles, with size %u bytes\n", bsize);
-	printf("Uncompressed write time: %-.6fs size: %u\n", wall[0], bsize);
-	printf("Compressed write time  : %-.6fs size: %d\n", wall[1], (int32_t)ret);
-	printf("Ratio                  : 1:%-.3f\n", (double)ret/(double)bsize);
+	printf("Uncompressed write time: %-.6fs size: %u, 1:%-.3f\n", wall[0], recsize[0], (double)recsize[0]/(double)bsize );
+	printf("LZO compressed write time  : %-.6fs size: %d, 1:%-.3f\n", wall[1], (int32_t)recsize[1], (double)recsize[1]/(double)bsize );
+	printf("LZ4 compressed write time  : %-.6fs size: %d, 1:%-.3f\n", wall[1], (int32_t)recsize[1], (double)recsize[1]/(double)bsize );
+	printf("BZ2 compressed write time  : %-.6fs size: %d, 1:%-.3f\n", wall[2], (int32_t)recsize[2], (double)recsize[2]/(double)bsize );
 
-	if ( wall[0] < wall[1] )
-		printf("You should run nfcapd without compression\n");
-	else
-		printf("You can run nfcapd with compression (-z)\n");
-	
 } // End of CheckCompression
 
 int main(int argc, char **argv) {
@@ -244,6 +226,14 @@ void *p;
 		exit(255);
     }
 
+	i = 1;
+	do {
+		if ( extension_descriptor[i].id != i ) {
+        	printf("**** FAILED **** Extension id %u != index %u\n", 
+				extension_descriptor[i].id, i);
+		}
+	} while ( extension_descriptor[++i].id );
+
 	p = (void *)c_record.data;
 	if (( (pointer_addr_t)p - (pointer_addr_t)&c_record ) != COMMON_RECORD_DATA_SIZE ) {
 		printf("*** common record size missmatch: expected %i, found: %llu\n",	
@@ -341,21 +331,49 @@ void *p;
 	ret = check_filter_block("proto 47", &flow_record, 1);
 	ret = check_filter_block("proto 42", &flow_record, 0);
 
+	flow_record.srcport = 0xaaaa;
 	flow_record.prot = IPPROTO_ICMP;
-	flow_record.dstport = 250; // -> icmp code 250
+	flow_record.dstport = 0xaffa; // -> icmp type 175, code 250
+	flow_record.icmp = flow_record.dstport;
+	flow_record.dstport = 0xbbbb;
+	ret = check_filter_block("icmp-type 175", &flow_record, 1);
+	ret = check_filter_block("icmp-type 176", &flow_record, 0);
 	ret = check_filter_block("icmp-code 250", &flow_record, 1);
 	ret = check_filter_block("icmp-code 251", &flow_record, 0);
-	flow_record.dstport = 3 << 8; // -> icmp type 8
+
+	if ( flow_record.icmp_type != 175 && flow_record.icmp_code != 250 ) {
+        printf("**** FAILED **** ICMP type check failed!\n" );
+        printf("ICMP type: %u, code: %u\n", flow_record.icmp_type, flow_record.icmp_code );
+		exit(255);
+	}
+
+	flow_record.dstport = 3 << 8; // -> icmp type 3
+	flow_record.icmp = flow_record.dstport;
+	if ( flow_record.icmp_type != 3 ) {
+        printf("**** FAILED **** ICMP type check failed!\n" );
+        printf("ICMP type: %u, code: %u\n", flow_record.icmp_type, flow_record.icmp_code );
+		exit(255);
+	}
 	ret = check_filter_block("icmp-type 3", &flow_record, 1);
 	ret = check_filter_block("icmp-type 4", &flow_record, 0);
 
+	flow_record.dstport = 8; // -> icmp code 8
+	flow_record.icmp = flow_record.dstport;
+	if ( flow_record.icmp_code != 8 ) {
+        printf("**** FAILED **** ICMP code check failed!\n" );
+        printf("ICMP type: %u, code: %u\n", flow_record.icmp_type, flow_record.icmp_code );
+		exit(255);
+	}
+	ret = check_filter_block("icmp-code 8", &flow_record, 1);
+	ret = check_filter_block("icmp-code 4", &flow_record, 0);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1234:5678", flow_record.v6.srcaddr);
-	inet_pton(PF_INET6, "fe80::1104:fedc:4321:8765", flow_record.v6.dstaddr);
-	flow_record.v6.srcaddr[0] = ntohll(flow_record.v6.srcaddr[0]);
-	flow_record.v6.srcaddr[1] = ntohll(flow_record.v6.srcaddr[1]);
-	flow_record.v6.dstaddr[0] = ntohll(flow_record.v6.dstaddr[0]);
-	flow_record.v6.dstaddr[1] = ntohll(flow_record.v6.dstaddr[1]);
+
+	inet_pton(PF_INET6, "fe80::2110:abcd:1234:5678", flow_record.V6.srcaddr);
+	inet_pton(PF_INET6, "fe80::1104:fedc:4321:8765", flow_record.V6.dstaddr);
+	flow_record.V6.srcaddr[0] = ntohll(flow_record.V6.srcaddr[0]);
+	flow_record.V6.srcaddr[1] = ntohll(flow_record.V6.srcaddr[1]);
+	flow_record.V6.dstaddr[0] = ntohll(flow_record.V6.dstaddr[0]);
+	flow_record.V6.dstaddr[1] = ntohll(flow_record.V6.dstaddr[1]);
 	ret = check_filter_block("src ip fe80::2110:abcd:1234:5678", &flow_record, 1);
 	ret = check_filter_block("src ip fe80::2110:abcd:1234:5679", &flow_record, 0);
 	ret = check_filter_block("src ip fe80::2111:abcd:1234:5678", &flow_record, 0);
@@ -372,39 +390,39 @@ void *p;
 	ret = check_filter_block("src ip in [fe80::2110:abcd:1234:5678]", &flow_record, 1);
 	ret = check_filter_block("src ip in [fe80::2110:abcd:1234:5679]", &flow_record, 0);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1234:0", flow_record.v6.srcaddr);
-	flow_record.v6.srcaddr[0] = ntohll(flow_record.v6.srcaddr[0]);
-	flow_record.v6.srcaddr[1] = ntohll(flow_record.v6.srcaddr[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1234:0", flow_record.V6.srcaddr);
+	flow_record.V6.srcaddr[0] = ntohll(flow_record.V6.srcaddr[0]);
+	flow_record.V6.srcaddr[1] = ntohll(flow_record.V6.srcaddr[1]);
 	ret = check_filter_block("src net fe80::2110:abcd:1234:0/112", &flow_record, 1);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1234:ffff", flow_record.v6.srcaddr);
-	flow_record.v6.srcaddr[0] = ntohll(flow_record.v6.srcaddr[0]);
-	flow_record.v6.srcaddr[1] = ntohll(flow_record.v6.srcaddr[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1234:ffff", flow_record.V6.srcaddr);
+	flow_record.V6.srcaddr[0] = ntohll(flow_record.V6.srcaddr[0]);
+	flow_record.V6.srcaddr[1] = ntohll(flow_record.V6.srcaddr[1]);
 	ret = check_filter_block("src net fe80::2110:abcd:1234:0/112", &flow_record, 1);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.v6.srcaddr);
-	flow_record.v6.srcaddr[0] = ntohll(flow_record.v6.srcaddr[0]);
-	flow_record.v6.srcaddr[1] = ntohll(flow_record.v6.srcaddr[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.V6.srcaddr);
+	flow_record.V6.srcaddr[0] = ntohll(flow_record.V6.srcaddr[0]);
+	flow_record.V6.srcaddr[1] = ntohll(flow_record.V6.srcaddr[1]);
 	ret = check_filter_block("src net fe80::2110:abcd:1234:0/112", &flow_record, 0);
 	ret = check_filter_block("src net fe80::0/16", &flow_record, 1);
 	ret = check_filter_block("src net fe81::0/16", &flow_record, 0);
 
-	flow_record.v6.srcaddr[0] = 0;
-	flow_record.v6.srcaddr[1] = 0;
+	flow_record.V6.srcaddr[0] = 0;
+	flow_record.V6.srcaddr[1] = 0;
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1234:0", flow_record.v6.dstaddr);
-	flow_record.v6.dstaddr[0] = ntohll(flow_record.v6.dstaddr[0]);
-	flow_record.v6.dstaddr[1] = ntohll(flow_record.v6.dstaddr[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1234:0", flow_record.V6.dstaddr);
+	flow_record.V6.dstaddr[0] = ntohll(flow_record.V6.dstaddr[0]);
+	flow_record.V6.dstaddr[1] = ntohll(flow_record.V6.dstaddr[1]);
 	ret = check_filter_block("dst net fe80::2110:abcd:1234:0/112", &flow_record, 1);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1234:ffff", flow_record.v6.dstaddr);
-	flow_record.v6.dstaddr[0] = ntohll(flow_record.v6.dstaddr[0]);
-	flow_record.v6.dstaddr[1] = ntohll(flow_record.v6.dstaddr[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1234:ffff", flow_record.V6.dstaddr);
+	flow_record.V6.dstaddr[0] = ntohll(flow_record.V6.dstaddr[0]);
+	flow_record.V6.dstaddr[1] = ntohll(flow_record.V6.dstaddr[1]);
 	ret = check_filter_block("dst net fe80::2110:abcd:1234:0/112", &flow_record, 1);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.v6.dstaddr);
-	flow_record.v6.dstaddr[0] = ntohll(flow_record.v6.dstaddr[0]);
-	flow_record.v6.dstaddr[1] = ntohll(flow_record.v6.dstaddr[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.V6.dstaddr);
+	flow_record.V6.dstaddr[0] = ntohll(flow_record.V6.dstaddr[0]);
+	flow_record.V6.dstaddr[1] = ntohll(flow_record.V6.dstaddr[1]);
 	ret = check_filter_block("dst net fe80::2110:abcd:1234:0/112", &flow_record, 0);
 	ret = check_filter_block("dst net fe80::0/16", &flow_record, 1);
 	ret = check_filter_block("not dst net fe80::0/16", &flow_record, 0);
@@ -415,13 +433,15 @@ void *p;
 	/* 172.32.7.16 => 0xac200710
 	 * 10.10.10.11 => 0x0a0a0a0b
 	 */
-	flow_record.v6.srcaddr[0] = 0;
-	flow_record.v6.srcaddr[1] = 0;
-	flow_record.v6.dstaddr[0] = 0;
-	flow_record.v6.dstaddr[1] = 0;
-	flow_record.v4.srcaddr = 0xac200710;
-	flow_record.v4.dstaddr = 0x0a0a0a0b;
+	flow_record.V6.srcaddr[0] = 0;
+	flow_record.V6.srcaddr[1] = 0;
+	flow_record.V6.dstaddr[0] = 0;
+	flow_record.V6.dstaddr[1] = 0;
+	flow_record.V4.srcaddr = 0xac200710;
+	flow_record.V4.dstaddr = 0x0a0a0a0b;
 	ret = check_filter_block("src ip 172.32.7.16", &flow_record, 1);
+	ret = check_filter_block("( src ip 172.32.7.16 ) %MyLabel", &flow_record, 1);
+	ret = check_filter_block("%MyLabel( src ip 172.32.7.16 )", &flow_record, 1);
 	ret = check_filter_block("src ip 172.32.7.15", &flow_record, 0);
 	ret = check_filter_block("dst ip 10.10.10.11", &flow_record, 1);
 	ret = check_filter_block("dst ip 10.10.10.10", &flow_record, 0);
@@ -578,53 +598,53 @@ void *p;
 	ret = check_filter_block("src ip 172.32.7.15 and dst ip 10.10.10.11", &flow_record, 0);
 	ret = check_filter_block("src ip 172.32.7.16 and dst ip 10.10.10.12", &flow_record, 0);
 
-	flow_record.v4.srcaddr = 0;
-	flow_record.v4.dstaddr = 0;
+	flow_record.V4.srcaddr = 0;
+	flow_record.V4.dstaddr = 0;
 
 	// 172.32.7.16 => 0xac200710
-	flow_record.ip_nexthop.v6[0] = 0;
-	flow_record.ip_nexthop.v6[1] = 0;
-	flow_record.ip_nexthop.v4 = 0xac200710;
+	flow_record.ip_nexthop.V6[0] = 0;
+	flow_record.ip_nexthop.V6[1] = 0;
+	flow_record.ip_nexthop.V4 = 0xac200710;
 	ret = check_filter_block("next ip 172.32.7.16", &flow_record, 1);
 	ret = check_filter_block("next ip 172.32.7.15", &flow_record, 0);
 	ret = check_filter_block("next ip in [172.32.7.16 fe80::2110:abcd:1235:ffff]", &flow_record, 1);
 	ret = check_filter_block("next ip in [172.32.7.15 fe80::2110:abcd:1235:ffff]", &flow_record, 0);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.ip_nexthop.v6);
-	flow_record.ip_nexthop.v6[0] = ntohll(flow_record.ip_nexthop.v6[0]);
-	flow_record.ip_nexthop.v6[1] = ntohll(flow_record.ip_nexthop.v6[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.ip_nexthop.V6);
+	flow_record.ip_nexthop.V6[0] = ntohll(flow_record.ip_nexthop.V6[0]);
+	flow_record.ip_nexthop.V6[1] = ntohll(flow_record.ip_nexthop.V6[1]);
 	ret = check_filter_block("next ip fe80::2110:abcd:1235:ffff", &flow_record, 1);
 	ret = check_filter_block("next ip in [172.32.7.16 fe80::2110:abcd:1235:ffff]", &flow_record, 1);
 	ret = check_filter_block("next ip in [172.32.7.16 fe80::2110:abcd:1235:fffe]", &flow_record, 0);
 	ret = check_filter_block("next ip fe80::2110:abcd:1235:fffe", &flow_record, 0);
 	ret = check_filter_block("next ip fe81::2110:abcd:1235:ffff", &flow_record, 0);
 
-	flow_record.ip_nexthop.v6[0] = 0;
-	flow_record.ip_nexthop.v6[1] = 0;
+	flow_record.ip_nexthop.V6[0] = 0;
+	flow_record.ip_nexthop.V6[1] = 0;
 
-	flow_record.bgp_nexthop.v6[0] = 0;
-	flow_record.bgp_nexthop.v6[1] = 0;
-	flow_record.bgp_nexthop.v4 = 0xac200710;
+	flow_record.bgp_nexthop.V6[0] = 0;
+	flow_record.bgp_nexthop.V6[1] = 0;
+	flow_record.bgp_nexthop.V4 = 0xac200710;
 	ret = check_filter_block("bgpnext ip 172.32.7.16", &flow_record, 1);
 	ret = check_filter_block("bgpnext ip 172.32.7.15", &flow_record, 0);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.bgp_nexthop.v6);
-	flow_record.bgp_nexthop.v6[0] = ntohll(flow_record.bgp_nexthop.v6[0]);
-	flow_record.bgp_nexthop.v6[1] = ntohll(flow_record.bgp_nexthop.v6[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.bgp_nexthop.V6);
+	flow_record.bgp_nexthop.V6[0] = ntohll(flow_record.bgp_nexthop.V6[0]);
+	flow_record.bgp_nexthop.V6[1] = ntohll(flow_record.bgp_nexthop.V6[1]);
 	ret = check_filter_block("bgpnext ip fe80::2110:abcd:1235:ffff", &flow_record, 1);
 	ret = check_filter_block("bgpnext ip fe80::2110:abcd:1235:fffe", &flow_record, 0);
 	ret = check_filter_block("bgpnext ip fe81::2110:abcd:1235:ffff", &flow_record, 0);
 
-	flow_record.ip_router.v6[0] = 0;
-	flow_record.ip_router.v6[1] = 0;
-	flow_record.ip_router.v4 = 0xac200720;
-	flow_record.ip_nexthop.v4 = 0xac200720;
+	flow_record.ip_router.V6[0] = 0;
+	flow_record.ip_router.V6[1] = 0;
+	flow_record.ip_router.V4 = 0xac200720;
+	flow_record.ip_nexthop.V4 = 0xac200720;
 	ret = check_filter_block("router ip 172.32.7.32", &flow_record, 1);
 	ret = check_filter_block("router ip 172.32.7.33", &flow_record, 0);
 
-	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.ip_router.v6);
-	flow_record.ip_router.v6[0] = ntohll(flow_record.ip_router.v6[0]);
-	flow_record.ip_router.v6[1] = ntohll(flow_record.ip_router.v6[1]);
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.ip_router.V6);
+	flow_record.ip_router.V6[0] = ntohll(flow_record.ip_router.V6[0]);
+	flow_record.ip_router.V6[1] = ntohll(flow_record.ip_router.V6[1]);
 	ret = check_filter_block("router ip fe80::2110:abcd:1235:ffff", &flow_record, 1);
 	ret = check_filter_block("router ip fe80::2110:abcd:1235:fffe", &flow_record, 0);
 	ret = check_filter_block("router ip fe81::2110:abcd:1235:ffff", &flow_record, 0);
@@ -986,5 +1006,237 @@ void *p;
 	ret = check_filter_block("sysid 44", &flow_record, 1);
 	ret = check_filter_block("sysid 45", &flow_record, 0);
 
+	// NSEL/ASA related tests
+#ifdef NSEL
+	flow_record.event = NSEL_EVENT_IGNORE;
+	flow_record.event_flag = FW_EVENT;
+	ret = check_filter_block("asa event ignore", &flow_record, 1);
+	ret = check_filter_block("asa event create", &flow_record, 0);
+	flow_record.event = NSEL_EVENT_CREATE;
+	ret = check_filter_block("asa event create", &flow_record, 1);
+	flow_record.event = NSEL_EVENT_DELETE;
+	ret = check_filter_block("asa event term", &flow_record, 1);
+	ret = check_filter_block("asa event delete", &flow_record, 1);
+	flow_record.event = NSEL_EVENT_DENIED;
+	ret = check_filter_block("asa event deny", &flow_record, 1);
+	ret = check_filter_block("asa event create", &flow_record, 0);
+	ret = check_filter_block("asa event 3", &flow_record, 1);
+	ret = check_filter_block("asa event > 2", &flow_record, 1);
+	ret = check_filter_block("asa event > 3", &flow_record, 0);
+
+	flow_record.fw_xevent = 1001;
+	ret = check_filter_block("asa event denied ingress", &flow_record, 1);
+	ret = check_filter_block("asa event denied egress", &flow_record, 0);
+	flow_record.fw_xevent = 1002;
+	ret = check_filter_block("asa event denied egress", &flow_record, 1);
+	flow_record.fw_xevent = 1003;
+	ret = check_filter_block("asa event denied interface", &flow_record, 1);
+	flow_record.fw_xevent = 1004;
+	ret = check_filter_block("asa event denied nosyn", &flow_record, 1);
+	ret = check_filter_block("asa event denied ingress", &flow_record, 0);
+	flow_record.event = NSEL_EVENT_CREATE;
+	ret = check_filter_block("asa event denied nosyn", &flow_record, 0);
+
+	ret = check_filter_block("asa xevent 1004", &flow_record, 1);
+	ret = check_filter_block("asa xevent < 1004", &flow_record, 0);
+	ret = check_filter_block("asa xevent > 1004", &flow_record, 0);
+
+	flow_record.xlate_src_ip.V6[0] = 0;
+	flow_record.xlate_src_ip.V6[1] = 0;
+	flow_record.xlate_src_ip.V4 = 0xac200710;
+	flow_record.xlate_dst_ip.V6[0] = 0;
+	flow_record.xlate_dst_ip.V6[1] = 0;
+	flow_record.xlate_dst_ip.V4 = 0x0a0a0a0b;
+	ret = check_filter_block("src xip 172.32.7.16", &flow_record, 1);
+	ret = check_filter_block("src xip 172.32.7.15", &flow_record, 0);
+	ret = check_filter_block("dst xip 10.10.10.11", &flow_record, 1);
+	ret = check_filter_block("dst xip 10.10.10.12", &flow_record, 0);
+	ret = check_filter_block("xip 172.32.7.16", &flow_record, 1);
+	ret = check_filter_block("xip 10.10.10.11", &flow_record, 1);
+	ret = check_filter_block("xip 172.32.7.15", &flow_record, 0);
+	ret = check_filter_block("xip 10.10.10.12", &flow_record, 0);
+	ret = check_filter_block("src xnet 172.32.7.0/24", &flow_record, 1);
+	ret = check_filter_block("src xnet 172.32.8.0/24", &flow_record, 0);
+	ret = check_filter_block("dst xnet 10.10.10.0/24", &flow_record, 1);
+	ret = check_filter_block("dst xnet 10.10.11.0/24", &flow_record, 0);
+	ret = check_filter_block("xnet 172.32.7.0/24", &flow_record, 1);
+	ret = check_filter_block("xnet 10.10.10.0/24", &flow_record, 1);
+
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.xlate_src_ip.V6);
+	flow_record.xlate_src_ip.V6[0] = ntohll(flow_record.xlate_src_ip.V6[0]);
+	flow_record.xlate_src_ip.V6[1] = ntohll(flow_record.xlate_src_ip.V6[1]);
+	ret = check_filter_block("src xip fe80::2110:abcd:1235:ffff", &flow_record, 1);
+	ret = check_filter_block("src xip fe80::2110:abcd:1235:fffe", &flow_record, 0);
+
+	flow_record.xlate_src_ip.V6[0] = 0;
+	flow_record.xlate_src_ip.V6[1] = 0;
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:fffe", flow_record.xlate_dst_ip.V6);
+	flow_record.xlate_dst_ip.V6[0] = ntohll(flow_record.xlate_dst_ip.V6[0]);
+	flow_record.xlate_dst_ip.V6[1] = ntohll(flow_record.xlate_dst_ip.V6[1]);
+	ret = check_filter_block("dst xip fe80::2110:abcd:1235:fffe", &flow_record, 1);
+	ret = check_filter_block("dst xip fe80::2110:abcd:1235:fffc", &flow_record, 0);
+	flow_record.xlate_src_ip.V6[0] = 0;
+	flow_record.xlate_src_ip.V6[1] = 0;
+	flow_record.xlate_dst_ip.V6[0] = 0;
+	flow_record.xlate_dst_ip.V6[1] = 0;
+
+	flow_record.xlate_src_port = 1023;
+	flow_record.xlate_dst_port = 32798;
+	ret = check_filter_block("src xport 1023", &flow_record, 1);
+	ret = check_filter_block("dst xport 32798", &flow_record, 1);
+	ret = check_filter_block("src xport > 1022", &flow_record, 1);
+	ret = check_filter_block("src xport < 1024", &flow_record, 1);
+	ret = check_filter_block("dst xport lt 32799", &flow_record, 1);
+	ret = check_filter_block("dst xport gt 32797", &flow_record, 1);
+	ret = check_filter_block("src xport > 1023", &flow_record, 0);
+	ret = check_filter_block("dst xport gt 32798", &flow_record, 0);
+	ret = check_filter_block("src xport 1022", &flow_record, 0);
+	ret = check_filter_block("dst xport 32797", &flow_record, 0);
+	flow_record.xlate_src_port = 0xffff;
+	flow_record.xlate_dst_port = 0xffff;
+
+	flow_record.ingress_acl_id[0] = 0xaabbcc;
+	flow_record.ingress_acl_id[1] = 0xbbccdd;
+	flow_record.ingress_acl_id[2] = 0xccddee;
+
+	flow_record.egress_acl_id[0] = 0x112233;
+	flow_record.egress_acl_id[1] = 0x223344;
+	flow_record.egress_acl_id[2] = 0x334455;
+
+	ret = check_filter_block("ingress ACL 0xaabbcc", &flow_record, 1);
+	flow_record.event = 255;
+
+
+	// NEL/NAT related tests
+	flow_record.event = NEL_EVENT_INVALID;
+	flow_record.event_flag = NAT_EVENT;
+	ret = check_filter_block("nat event invalid", &flow_record, 1);
+	ret = check_filter_block("nat event add", &flow_record, 0);
+	flow_record.event = NEL_EVENT_ADD;
+	ret = check_filter_block("nat event add", &flow_record, 1);
+	flow_record.event = NEL_EVENT_DELETE;
+	ret = check_filter_block("nat event delete", &flow_record, 1);
+	ret = check_filter_block("nat event add", &flow_record, 0);
+	ret = check_filter_block("nat event 2", &flow_record, 1);
+	ret = check_filter_block("nat event > 1", &flow_record, 1);
+	ret = check_filter_block("nat event > 2", &flow_record, 0);
+	flow_record.event = 255;
+
+	flow_record.ingress_vrfid = 0xAAAA;
+	ret = check_filter_block("ingress vrf 0xAAAA", &flow_record, 1);
+	ret = check_filter_block("ingress vrf 100", &flow_record, 0);
+
+	flow_record.egress_vrfid = 0xBBBB;
+	ret = check_filter_block("egress vrf 0xBBBB", &flow_record, 1);
+	ret = check_filter_block("egress vrf 0xAAAA", &flow_record, 0);
+
+	flow_record.block_start = 1111;
+	ret = check_filter_block("pblock start 1111", &flow_record, 1);
+	ret = check_filter_block("pblock start 2222", &flow_record, 0);
+
+	flow_record.block_end = 2222;
+	ret = check_filter_block("pblock end 2222", &flow_record, 1);
+	ret = check_filter_block("pblock end 3333", &flow_record, 0);
+
+	flow_record.block_step = 3333;
+	ret = check_filter_block("pblock step 3333", &flow_record, 1);
+	ret = check_filter_block("pblock step 4444", &flow_record, 0);
+
+	flow_record.block_size = 4444;
+	ret = check_filter_block("pblock size 4444", &flow_record, 1);
+	ret = check_filter_block("pblock size 5555", &flow_record, 0);
+
+	flow_record.srcport = 63;
+	flow_record.dstport = 255;
+	ret = check_filter_block("src port in pblock", &flow_record, 0);
+	ret = check_filter_block("dst port in pblock", &flow_record, 0);
+	ret = check_filter_block("port in pblock", &flow_record, 0);
+
+	flow_record.srcport = 1110;
+	ret = check_filter_block("src port in pblock", &flow_record, 0);
+	ret = check_filter_block("port in pblock", &flow_record, 0);
+
+	flow_record.srcport = 1111;
+	ret = check_filter_block("src port in pblock", &flow_record, 1);
+	ret = check_filter_block("port in pblock", &flow_record, 1);
+
+	flow_record.srcport = 2222;
+	ret = check_filter_block("src port in pblock", &flow_record, 1);
+	ret = check_filter_block("port in pblock", &flow_record, 1);
+
+	flow_record.srcport = 2223;
+	ret = check_filter_block("src port in pblock", &flow_record, 0);
+	ret = check_filter_block("port in pblock", &flow_record, 0);
+
+	flow_record.dstport = 1110;
+	ret = check_filter_block("src port in pblock", &flow_record, 0);
+	ret = check_filter_block("dst port in pblock", &flow_record, 0);
+	ret = check_filter_block("port in pblock", &flow_record, 0);
+
+	flow_record.dstport = 1111;
+	ret = check_filter_block("dst port in pblock", &flow_record, 1);
+	ret = check_filter_block("port in pblock", &flow_record, 1);
+exit(0);
+
+	flow_record.dstport = 2222;
+	ret = check_filter_block("dst port in pblock", &flow_record, 1);
+	ret = check_filter_block("port in pblock", &flow_record, 1);
+
+	flow_record.dstport = 2223;
+	ret = check_filter_block("dst port in pblock", &flow_record, 0);
+	ret = check_filter_block("port in pblock", &flow_record, 0);
+
+	flow_record.srcport = 1234;
+	flow_record.srcport = 2134;
+	ret = check_filter_block("src port in pblock", &flow_record, 1);
+	ret = check_filter_block("dst port in pblock", &flow_record, 1);
+	ret = check_filter_block("port in pblock", &flow_record, 1);
+
+	flow_record.xlate_src_port = 1023;
+	flow_record.xlate_dst_port = 32798;
+	ret = check_filter_block("src nport 1023", &flow_record, 1);
+	ret = check_filter_block("dst nport 32798", &flow_record, 1);
+	ret = check_filter_block("src nport > 1022", &flow_record, 1);
+	ret = check_filter_block("src nport < 1024", &flow_record, 1);
+	ret = check_filter_block("dst nport lt 32799", &flow_record, 1);
+	ret = check_filter_block("dst nport gt 32797", &flow_record, 1);
+	ret = check_filter_block("src nport > 1023", &flow_record, 0);
+	ret = check_filter_block("dst nport gt 32798", &flow_record, 0);
+	ret = check_filter_block("src nport 1022", &flow_record, 0);
+	ret = check_filter_block("dst nport 32797", &flow_record, 0);
+	flow_record.xlate_src_port = 0xffff;
+	flow_record.xlate_dst_port = 0xffff;
+
+	flow_record.xlate_src_ip.V6[0] = 0;
+	flow_record.xlate_src_ip.V6[1] = 0;
+	flow_record.xlate_src_ip.V4 = 0xac200710;
+	flow_record.xlate_dst_ip.V6[0] = 0;
+	flow_record.xlate_dst_ip.V6[1] = 0;
+	flow_record.xlate_dst_ip.V4 = 0x0a0a0a0b;
+	ret = check_filter_block("src nip 172.32.7.16", &flow_record, 1);
+	ret = check_filter_block("src nip 172.32.7.15", &flow_record, 0);
+	ret = check_filter_block("dst nip 10.10.10.11", &flow_record, 1);
+	ret = check_filter_block("dst nip 10.10.10.12", &flow_record, 0);
+	ret = check_filter_block("nip 172.32.7.16", &flow_record, 1);
+	ret = check_filter_block("nip 10.10.10.11", &flow_record, 1);
+	ret = check_filter_block("nip 172.32.7.15", &flow_record, 0);
+	ret = check_filter_block("nip 10.10.10.12", &flow_record, 0);
+
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:ffff", flow_record.xlate_src_ip.V6);
+	flow_record.xlate_src_ip.V6[0] = ntohll(flow_record.xlate_src_ip.V6[0]);
+	flow_record.xlate_src_ip.V6[1] = ntohll(flow_record.xlate_src_ip.V6[1]);
+	ret = check_filter_block("src nip fe80::2110:abcd:1235:ffff", &flow_record, 1);
+	ret = check_filter_block("src nip fe80::2110:abcd:1235:fffe", &flow_record, 0);
+
+	flow_record.xlate_src_ip.V6[0] = 0;
+	flow_record.xlate_src_ip.V6[1] = 0;
+	inet_pton(PF_INET6, "fe80::2110:abcd:1235:fffe", flow_record.xlate_dst_ip.V6);
+	flow_record.xlate_dst_ip.V6[0] = ntohll(flow_record.xlate_dst_ip.V6[0]);
+	flow_record.xlate_dst_ip.V6[1] = ntohll(flow_record.xlate_dst_ip.V6[1]);
+	ret = check_filter_block("dst nip fe80::2110:abcd:1235:fffe", &flow_record, 1);
+	ret = check_filter_block("dst nip fe80::2110:abcd:1235:fffc", &flow_record, 0);
+
+#endif
+
 	return 0;
 }
