@@ -79,12 +79,13 @@ static char		**IdentList;
 static void UpdateList(uint32_t a, uint32_t b);
 
 /* flow processing functions */
-static inline uint64_t pps_function(uint64_t *data);
-static inline uint64_t bps_function(uint64_t *data);
-static inline uint64_t bpp_function(uint64_t *data);
-static inline uint64_t duration_function(uint64_t *data);
-static inline uint64_t mpls_eos_function(uint64_t *data);
-static inline uint64_t mpls_any_function(uint64_t *data);
+static inline void pps_function(uint64_t *record_data, uint64_t *comp_values);
+static inline void bps_function(uint64_t *record_data, uint64_t *comp_values);
+static inline void bpp_function(uint64_t *record_data, uint64_t *comp_values);
+static inline void duration_function(uint64_t *record_data, uint64_t *comp_values);
+static inline void mpls_eos_function(uint64_t *record_data, uint64_t *comp_values);
+static inline void mpls_any_function(uint64_t *record_data, uint64_t *comp_values);
+static inline void pblock_function(uint64_t *record_data, uint64_t *comp_values);
 
 /* 
  * flow processing function table:
@@ -100,7 +101,8 @@ static struct flow_procs_map_s {
 	{"bpp",			bpp_function},
 	{"duration",	duration_function},
 	{"mpls eos",	mpls_eos_function},
- 	{"mpls any",	mpls_any_function},
+	{"mpls any",	mpls_any_function},
+ 	{"pblock", 		pblock_function},
 	{NULL,			NULL}
 };
 
@@ -242,6 +244,7 @@ uint32_t	NewBlock(uint32_t offset, uint64_t mask, uint64_t value, uint16_t comp,
 	FilterTree[n].comp 		= comp;
 	FilterTree[n].function 	= flow_procs_map[function].function;
 	FilterTree[n].fname 	= flow_procs_map[function].name;
+	FilterTree[n].label 	= NULL;
 	FilterTree[n].data 		= data;
 	if ( comp > 0 || function > 0 )
 		Extended = 1;
@@ -381,15 +384,17 @@ void DumpList(FilterEngine_data_t *args) {
 
 	for (i=1; i<NumBlocks; i++ ) {
 		if ( args->filter[i].invert )
-			printf("Index: %u, Offset: %u, Mask: %.16llx, Value: %.16llx, Superblock: %u, Numblocks: %u, !OnTrue: %u, !OnFalse: %u Comp: %u Function: %s\n",
+			printf("Index: %u, Offset: %u, Mask: %.16llx, Value: %.16llx, Superblock: %u, Numblocks: %u, !OnTrue: %u, !OnFalse: %u Comp: %u Function: %s, Label: %s\n",
 				i, args->filter[i].offset, (unsigned long long)args->filter[i].mask, 
 				(unsigned long long)args->filter[i].value, args->filter[i].superblock, 
-				args->filter[i].numblocks, args->filter[i].OnTrue, args->filter[i].OnFalse, args->filter[i].comp, args->filter[i].fname);
+				args->filter[i].numblocks, args->filter[i].OnTrue, args->filter[i].OnFalse, 
+				args->filter[i].comp, args->filter[i].fname, args->filter[i].label ? args->filter[i].label : "<none>");
 		else 
-			printf("Index: %u, Offset: %u, Mask: %.16llx, Value: %.16llx, Superblock: %u, Numblocks: %u, OnTrue: %u, OnFalse: %u Comp: %u Function: %s\n",
+			printf("Index: %u, Offset: %u, Mask: %.16llx, Value: %.16llx, Superblock: %u, Numblocks: %u, OnTrue: %u, OnFalse: %u Comp: %u Function: %s, Label: %s\n",
 				i, args->filter[i].offset, (unsigned long long)args->filter[i].mask, 
 				(unsigned long long)args->filter[i].value, args->filter[i].superblock, 
-				args->filter[i].numblocks, args->filter[i].OnTrue, args->filter[i].OnFalse, args->filter[i].comp, args->filter[i].fname);
+				args->filter[i].numblocks, args->filter[i].OnTrue, args->filter[i].OnFalse, 
+				args->filter[i].comp, args->filter[i].fname, args->filter[i].label ? args->filter[i].label : "<none>");
 		if ( args->filter[i].OnTrue > (memblocks * MAXBLOCKS) || args->filter[i].OnFalse > (memblocks * MAXBLOCKS) ) {
 			fprintf(stderr, "Tree pointer out of range for index %u. *** ABORT ***\n", i);
 			exit(255);
@@ -426,6 +431,7 @@ int RunFilter(FilterEngine_data_t *args) {
 uint32_t	index, offset;
 int	evaluate, invert;
 
+	args->label = NULL;
 	index = args->StartNode;
 	evaluate = 0;
 	invert = 0;
@@ -442,9 +448,10 @@ int	evaluate, invert;
 /* extended filter engine */
 int RunExtendedFilter(FilterEngine_data_t *args) {
 uint32_t	index, offset; 
-uint64_t	value;
+uint64_t	comp_value[2];
 int	evaluate, invert;
 
+	args->label = NULL;
 	index = args->StartNode;
 	evaluate = 0;
 	invert = 0;
@@ -452,30 +459,30 @@ int	evaluate, invert;
 		offset   = args->filter[index].offset;
 		invert   = args->filter[index].invert;
 
-		if (args->filter[index].function == NULL)
-			value = args->nfrecord[offset] & args->filter[index].mask;
-		else
-			value = args->filter[index].function(args->nfrecord);
+		comp_value[0] = args->nfrecord[offset] & args->filter[index].mask;
+		comp_value[1] = args->filter[index].value;
+
+		if (args->filter[index].function != NULL)
+			args->filter[index].function(args->nfrecord, comp_value);
 
 		switch (args->filter[index].comp) {
 			case CMP_EQ:
-				evaluate = value == args->filter[index].value;
+				evaluate = comp_value[0] == comp_value[1];
 				break;
 			case CMP_GT:
-				evaluate = value > args->filter[index].value;
+				evaluate = comp_value[0] > comp_value[1];
 				break;
 			case CMP_LT:
-				evaluate = value < args->filter[index].value;
+				evaluate = comp_value[0] < comp_value[1];
 				break;
 			case CMP_IDENT:
-				value = args->filter[index].value;
-				evaluate = strncmp(CurrentIdent, args->IdentList[value], IDENTLEN) == 0 ;
+				evaluate = strncmp(CurrentIdent, args->IdentList[comp_value[1]], IDENTLEN) == 0 ;
 				break;
 			case CMP_FLAGS:
 				if ( invert )
-					evaluate = value > 0;
+					evaluate = comp_value[0] > 0;
 				else
-					evaluate = value == args->filter[index].value;
+					evaluate = comp_value[0] == comp_value[1];
 				break;
 			case CMP_IPLIST: {
 				struct IPListNode find;
@@ -487,17 +494,45 @@ int	evaluate, invert;
 				break;
 			case CMP_ULLIST: {
 				struct ULongListNode find;
-				find.value = value;
+				find.value = comp_value[0];
 				evaluate = RB_FIND(ULongtree, args->filter[index].data, &find ) != NULL; }
 				break;
 		}
 
-		index = evaluate ? args->filter[index].OnTrue : args->filter[index].OnFalse;
+		/*
+		 * Label evaluation:
+		 * A flow gets labeled, if one filter expression has a label assigned and
+		 * that filter expression is in the 'true' path of the tree, resulting
+		 * to a final match. If subsequent expressions in the same path evaluate
+		 * to false, the label is cleared again.
+		 * In case of multiple labels in a true patch, the last seen label wins.
+		 */
+		if ( evaluate ) {
+			// if filter expression has a label assigned, copy that
+			if ( args->filter[index].label ) {
+				args->label = args->filter[index].label;
+			}
+			index = args->filter[index].OnTrue;
+		} else {
+			// filter expression does not match - clear previous label if abailable
+			if ( args->label )
+				args->label = NULL;
+			index = args->filter[index].OnFalse;
+		}
+		// index = evaluate ? args->filter[index].OnTrue : args->filter[index].OnFalse;
 	}
 	return invert ? !evaluate : evaluate;
 
 } /* End of RunExtendedFilter */
 
+void AddLabel(uint32_t index, char *label) {
+
+	FilterTree[index].label = strdup(label);
+	//Evaluation requires extended engine
+	Extended = 1;
+
+} // End of AddLabel
+
 uint32_t AddIdent(char *Ident) {
 uint32_t	num;
 
@@ -534,90 +569,96 @@ uint32_t	num;
 
 /* record processing functions */
 
-static inline uint64_t duration_function(uint64_t *data) {
-master_record_t *record;
-uint64_t		duration;
+static inline void duration_function(uint64_t *record_data, uint64_t *comp_values) {
+master_record_t *record = (master_record_t *)record_data;
 
-	record = (master_record_t *)data;
 	/* duration in msec */
-	duration = 1000*(record->last - record->first) + record->msec_last - record->msec_first;
-
-	return duration;
+	comp_values[0] = 1000*(record->last - record->first) + record->msec_last - record->msec_first;
 
 } // End of duration_function
 
-static inline uint64_t pps_function(uint64_t *data) {
-master_record_t *record;
+static inline void pps_function(uint64_t *record_data, uint64_t *comp_values) {
+master_record_t *record = (master_record_t *)record_data;
 uint64_t		duration;
 
-	record = (master_record_t *)data;
 	/* duration in msec */
 	duration = 1000*(record->last - record->first) + record->msec_last - record->msec_first;
 	if ( duration == 0 )
-		return 0;
+		comp_values[0] = 0;
 	else 
-		return ( 1000LL * record->dPkts ) / duration;
+		comp_values[0] = ( 1000LL * record->dPkts ) / duration;
 
 } // End of pps_function
 
-static inline uint64_t bps_function(uint64_t *data) {
-master_record_t *record;
+static inline void bps_function(uint64_t *record_data, uint64_t *comp_values) {
+master_record_t *record = (master_record_t *)record_data;
 uint64_t		duration;
 
-	record = (master_record_t *)data;
 	/* duration in msec */
 	duration = 1000*(record->last - record->first) + record->msec_last - record->msec_first;
 	if ( duration == 0 )
-		return 0;
+		comp_values[0] = 0;
 	else 
-		return ( 8000LL * record->dOctets ) / duration;	/* 8 bits per Octet - x 1000 for msec */
+		comp_values[0] = ( 8000LL * record->dOctets ) / duration;	/* 8 bits per Octet - x 1000 for msec */
 
 } // End of bps_function
 
-static inline uint64_t bpp_function(uint64_t *data) {
-master_record_t 	*record;
+static inline void bpp_function(uint64_t *record_data, uint64_t *comp_values) {
+master_record_t *record = (master_record_t *)record_data;
 
-	record = (master_record_t *)data;
-	return record->dPkts ? record->dOctets / record->dPkts : 0;
+	comp_values[0] = record->dPkts ? record->dOctets / record->dPkts : 0;
 
 } // End of bpp_function
 
-static inline uint64_t mpls_eos_function(uint64_t *data) {
-master_record_t 	*record;
+static inline void mpls_eos_function(uint64_t *record_data, uint64_t *comp_values) {
+master_record_t *record = (master_record_t *)record_data;
 int i;
 
-	record = (master_record_t *)data;
-
 	// search for end of MPLS stack label
 	for (i=0; i<10; i++ ) {
 		if ( record->mpls_label[i] & 1 ) {
 			// End of stack found -> mask exp and eos bits
-			return record->mpls_label[i] & 0x00FFFFF0;
+			comp_values[0] = record->mpls_label[i] & 0x00FFFFF0;
+			return;
 		}
 	}
 
 	// trick filter to fail with an invalid mpls label value
-	return 0xFF000000;
+	comp_values[0] = 0xFF000000;
 
 } // End of mpls_eos_function
 
-static inline uint64_t mpls_any_function(uint64_t *data) {
-master_record_t *record;
+static inline void mpls_any_function(uint64_t *record_data, uint64_t *comp_values) {
+master_record_t *record = (master_record_t *)record_data;
 int i;
 
-	record = (master_record_t *)data;
-
 	// search for end of MPLS stack label
 	for (i=0; i<10; i++ ) {
 		if ( (record->mpls_label[i] & 1) == 1 ) {
 			// End of stack found -> mask exp and eos bits
-			return record->mpls_label[i] & 0x00FFFFF0;
+			comp_values[0] = record->mpls_label[i] & 0x00FFFFF0;
+			return;
 		}
 	}
 
 	// trick filter to fail with an invalid mpls label value
-	return 0xFF000000;
+	comp_values[0] = 0xFF000000;
 
 } // End of mpls_eos_function
 
+static inline void pblock_function(uint64_t *record_data, uint64_t *comp_values) {
+#ifdef NSEL
+master_record_t *record = (master_record_t *)record_data;
+	comp_values[0] = comp_values[0] >> comp_values[1];
+	if ( (comp_values[0] >= record->block_start) && (comp_values[0] <= record->block_end) ) {
+		comp_values[1] = comp_values[0];
+	} else {
+		// force "not equal"
+		comp_values[1] = comp_values[0] + 1;
+	}
+#else
+	comp_values[1] = 0;
+#endif
+
+} // End of pblock_function
 
