@@ -1,37 +1,34 @@
 /*
- *  Copyright (c) 2017, Peter Haag
- *  Copyright (c) 2016, Peter Haag
- *  Copyright (c) 2014, Peter Haag
+ *  Copyright (c) 2014-2019, Peter Haag
  *  All rights reserved.
- *  
- *  Redistribution and use in source and binary forms, with or without 
+ *
+ *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
- *  
- *   * Redistributions of source code must retain the above copyright notice, 
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
  *	 this list of conditions and the following disclaimer.
- *   * Redistributions in binary form must reproduce the above copyright notice, 
- *	 this list of conditions and the following disclaimer in the documentation 
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *	 this list of conditions and the following disclaimer in the documentation
  *	 and/or other materials provided with the distribution.
- *   * Neither the name of the author nor the names of its contributors may be 
- *	 used to endorse or promote products derived from this software without 
+ *   * Neither the name of the author nor the names of its contributors may be
+ *	 used to endorse or promote products derived from this software without
  *	 specific prior written permission.
- *  
- *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
- *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
- *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
- *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
- *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
- *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
- *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
- *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  *  POSSIBILITY OF SUCH DAMAGE.
- *  
- *  
+ *
  */
 
-#ifdef HAVE_CONFIG_H 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -76,23 +73,15 @@
 #include "util.h"
 #include "netflow_pcap.h"
 
-static inline void ProcessTCPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
-static inline void ProcessUDPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
-static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
-static inline void ProcessOtherFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
 struct pcap_timeval {
-	int32_t tv_sec;	   /* seconds */
-	int32_t tv_usec;	  /* microseconds */
+	int32_t tv_sec;   /* seconds */
+	int32_t tv_usec;  /* microseconds */
 };
 
 struct pcap_sf_pkthdr {
 	struct pcap_timeval ts; /* time stamp */
-	uint32_t	caplen;	 /* length of portion present */
-	uint32_t	len;		/* length this packet (off wire) */
+	uint32_t	caplen;	    /* length of portion present */
+	uint32_t	len;        /* length this packet (off wire) */
 };
 
 typedef struct vlan_hdr_s {
@@ -105,7 +94,13 @@ typedef struct gre_hdr_s {
   uint16_t type;
 } gre_hdr_t;
 
-int lock_sync = 0;
+static inline void ProcessTCPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
+
+static inline void ProcessUDPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
+
+static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
+
+static inline void ProcessOtherFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
 
 pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 
@@ -113,7 +108,7 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 		// Create struct
 		pcapfile = calloc(1, sizeof(pcapfile_t));
 		if ( !pcapfile ) {
-			LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 			return NULL;
 		}
 		pthread_mutex_init(&pcapfile->m_pbuff, NULL);
@@ -122,14 +117,14 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 		pcapfile->data_buffer = malloc(BUFFSIZE);
 		if ( !pcapfile->data_buffer ) {
 			free(pcapfile);
-			LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 			return NULL;
 		}
 		pcapfile->alternate_buffer = malloc(BUFFSIZE);
 		if ( !pcapfile->data_buffer ) {
 			free(pcapfile->data_buffer);
 			free(pcapfile);
-			LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 			return NULL;
 		}
 		pcapfile->data_ptr		 = pcapfile->data_buffer;
@@ -148,7 +143,7 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 			pcapfile->pfd = fileno((FILE *)pcapfile->pd);
 			return pcapfile;
 		}
-	} else 
+	} else
 		return pcapfile;
 
 } // End of OpenNewPcapFile
@@ -157,7 +152,7 @@ int ClosePcapFile(pcapfile_t *pcapfile) {
 int err = 0;
 
 	if ( fclose((FILE *)pcapfile->pd) < 0 ) {
-		LogError("close() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		LogError("close() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 		err = errno;
 	}
 	pcapfile->pfd = -1;
@@ -196,7 +191,7 @@ void *_b;
 		if( pcap_stats(pcapfile->p, &p_stat) < 0) {
 			LogError("pcap_stats() failed: %s", pcap_geterr(pcapfile->p));
 		} else {
-			LogInfo("Packets received: %u, dropped: %u, dropped by interface: %u ", 
+			LogInfo("Packets received: %u, dropped: %u, dropped by interface: %u ",
 				p_stat.ps_recv, p_stat.ps_drop, p_stat.ps_ifdrop );
 		}
 	}
@@ -252,7 +247,7 @@ size_t	size = sizeof(struct pcap_sf_pkthdr) + h->caplen;
 
 } // End of PcapDump
 
-static inline void ProcessTCPFlow(FlowSource_t	*fs, struct FlowNode *NewNode ) {
+static inline void ProcessTCPFlow(FlowSource_t *fs, struct FlowNode *NewNode ) {
 struct FlowNode *Node;
 
 	assert(NewNode->memflag == NODE_IN_USE);
@@ -267,14 +262,7 @@ struct FlowNode *Node;
 			// flush node
 			if ( StorePcapFlow(fs, NewNode) ) {
 				Remove_Node(NewNode);
-			} 
-		}
-
-		if ( !CacheCheck() ) {
-			uint32_t NumFlows;
-			LogError("Node cache exhausted! - Immediate flush - increase flow cache!!");	
-			NumFlows  = Flush_FlowTree(fs);
-			LogError("Flushed flows: %u", NumFlows);	
+			}
 		}
 
 		if ( Link_RevNode(NewNode)) {
@@ -296,8 +284,8 @@ struct FlowNode *Node;
 	// update existing flow
 	Node->flags |= NewNode->flags;
 	Node->packets++;
-	Node->bytes += NewNode->bytes; 
-	Node->t_last = NewNode->t_last; 
+	Node->bytes += NewNode->bytes;
+	Node->t_last = NewNode->t_last;
 	dbg_printf("Existing TCP flow: Packets: %u, Bytes: %u\n", Node->packets, Node->bytes);
 
 	if ( NewNode->fin == FIN_NODE) {
@@ -305,11 +293,11 @@ struct FlowNode *Node;
 		Node->fin = FIN_NODE;
 		if ( StorePcapFlow(fs, Node) ) {
 			Remove_Node(Node);
-		} 
+		}
 	} else {
 		Free_Node(NewNode);
 	}
- 
+
 
 } // End of ProcessTCPFlow
 
@@ -317,7 +305,7 @@ static inline void ProcessUDPFlow(FlowSource_t	*fs, struct FlowNode *NewNode ) {
 struct FlowNode *Node;
 
 	assert(NewNode->memflag == NODE_IN_USE);
-	// Flush DNS queries directly 
+	// Flush DNS queries directly
 	if ( NewNode->src_port == 53 || NewNode->dst_port == 53 ) {
 		StorePcapFlow(fs, NewNode);
 		Free_Node(NewNode);
@@ -330,14 +318,13 @@ struct FlowNode *Node;
 	if ( Node == NULL ) {
 		dbg_printf("New UDP flow: Packets: %u, Bytes: %u\n", NewNode->packets, NewNode->bytes);
 		return;
-	} 
+	}
 	assert(Node->memflag == NODE_IN_USE);
 
 	// update existing flow
 	Node->packets++;
-	Node->bytes += NewNode->bytes; 
-	Node->t_last = NewNode->t_last; 
-
+	Node->bytes += NewNode->bytes;
+	Node->t_last = NewNode->t_last;
 	dbg_printf("Existing UDP flow: Packets: %u, Bytes: %u\n", Node->packets, Node->bytes);
 
 	Free_Node(NewNode);
@@ -346,7 +333,7 @@ struct FlowNode *Node;
 
 static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode ) {
 
-	// Flush ICMP directly 
+	// Flush ICMP directly
 	StorePcapFlow(fs, NewNode);
 	dbg_printf("Flush ICMP flow: Packets: %u, Bytes: %u\n", NewNode->packets, NewNode->bytes);
 
@@ -378,9 +365,8 @@ void ProcessFlowNode(FlowSource_t *fs, struct FlowNode *node) {
 		case IPPROTO_ICMPV6:
 			ProcessICMPFlow(fs, node);
 			break;
-		default: 
+		default:
 			ProcessOtherFlow(fs, node);
-
 	}
 
 } // End of ProcessFlowNode
@@ -391,10 +377,8 @@ struct ip 	  *ip;
 void		  *payload, *defragmented;
 uint32_t	  size_ip, offset, data_len, payload_len, bytes;
 uint16_t	  version, ethertype, proto;
-#ifdef DEVEL
 char		  s1[64];
 char		  s2[64];
-#endif
 static unsigned pkg_cnt = 0;
 
 	pkg_cnt++;
@@ -402,6 +386,7 @@ static unsigned pkg_cnt = 0;
 
 	pcap_dev->proc_stat.packets++;
 	offset = pcap_dev->linkoffset;
+	defragmented = NULL;
 
 	Node = New_Node();
 	if ( !Node ) {
@@ -423,10 +408,11 @@ static unsigned pkg_cnt = 0;
 				case 0x800:	 // IPv4
 				case 0x86DD: // IPv6
 					break;
-				case 0x8100: {	// VLAN 
+				case 0x8100: {	// VLAN
 					do {
 						vlan_hdr_t *vlan_hdr = (vlan_hdr_t *)(data + offset);  // offset points to end of link layer
-						dbg_printf("VLAN ID: %u, type: 0x%x\n", ntohs(vlan_hdr->vlan_id), ntohs(vlan_hdr->type) );
+						dbg_printf("VLAN ID: %u, type: 0x%x\n",
+							ntohs(vlan_hdr->vlan_id), ntohs(vlan_hdr->type) );
 						ethertype = ntohs(vlan_hdr->type);
 /*
 pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
@@ -442,7 +428,7 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 				case 0x806:	 // skip ARP
 					// silently skip ARP
 					pcap_dev->proc_stat.skipped++;
-					return;
+					goto END_FUNC;
 					break;
 				case 0x26:	 // ?? multicast router termination ??
 				case 0x4305: // B.A.T.M.A.N. BATADV
@@ -451,6 +437,7 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 				case 0x88cc: // CISCO LLDP
 				case 0x9000: // Loop
 				case 0x880b: // PPP - rfc 7042
+				case 0x6558: // Ethernet Bridge
 					pcap_dev->proc_stat.skipped++;
 					if ( Node->proto ) {
 						// if it's an encap which we do not understand yet - push tunnel
@@ -460,21 +447,21 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 						dbg_printf("Skip Ethertype 0x%x", ethertype);
 						Free_Node(Node);
 					}
-					return;
+					goto END_FUNC;
 					break;
 				default:
 					pcap_dev->proc_stat.unknown++;
-					LogError("Unsupported link type: 0x%x, packet: %u", ethertype, pkg_cnt);
+					LogInfo("Unsupported link type: 0x%x, packet: %u", ethertype, pkg_cnt);
 					Free_Node(Node);
-					return;
+					goto END_FUNC;
 			}
 	}
 
 	if (hdr->caplen < offset) {
 		pcap_dev->proc_stat.short_snap++;
-		LogError("Short packet: %u/%u", hdr->caplen, offset);
+		LogInfo("Short packet: %u/%u", hdr->caplen, offset);
 		Free_Node(Node);
-		return;
+		goto END_FUNC;
 	}
 
 	Node->t_first.tv_sec = hdr->ts.tv_sec;
@@ -485,7 +472,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 	data	 = data + offset;
 	data_len = hdr->caplen - offset;
 	offset	 = 0;
-	defragmented = NULL;
 
 	// IP decoding
 	REDO_IPPROTO:
@@ -507,11 +493,11 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		offset = size_ip;	// offset point to end of IP header
 
 		if ( data_len < size_ip ) {
-			LogError("Packet: %u Length error: data_len: %u < size IPV6: %u, captured: %u, hdr len: %u", 
+			LogInfo("Packet: %u Length error: data_len: %u < size IPV6: %u, captured: %u, hdr len: %u",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
 			Free_Node(Node);
-			return;
+			goto END_FUNC;
 		}
 
 		// XXX Extension headers not processed
@@ -545,18 +531,17 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		offset = size_ip;	// offset point to end of IP header
 
 		if ( data_len < size_ip ) {
-			LogError("Packet: %u Length error: data_len: %u < size IPV4: %u, captured: %u, hdr len: %u", 
+			LogInfo("Packet: %u Length error: data_len: %u < size IPV4: %u, captured: %u, hdr len: %u",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
 			Free_Node(Node);
-			return;
+			goto END_FUNC;
 		}
 
 		payload_len = ntohs(ip->ip_len);
-		dbg_printf("size IP hader: %u, len: %u, %u\n", size_ip, ip->ip_len, payload_len);
+		dbg_printf("size IP hader: %u, len: %u\n", size_ip, payload_len);
 
 		payload_len -= size_ip;	// ajust length compatibel IPv6
-		bytes 		= payload_len;
 		payload = (void *)ip + size_ip;
 		proto   = ip->ip_p;
 
@@ -575,23 +560,29 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint16_t ip_id = ntohs(ip->ip_id);
 #ifdef DEVEL
 			if ( frag_offset == 0 )
-				printf("Fragmented packet: first segement: ip_off: %u, frag_offset: %u\n", ip_off, frag_offset);
+				printf("Fragmented packet: first segement: ip_off: %u, frag_offset: %u\n",
+					ip_off, frag_offset);
 			if (( ip_off & IP_MF ) && frag_offset )
-				printf("Fragmented packet: middle segement: ip_off: %u, frag_offset: %u\n", ip_off, frag_offset);
+				printf("Fragmented packet: middle segement: ip_off: %u, frag_offset: %u\n",
+					ip_off, frag_offset);
 			if (( ip_off & IP_MF ) == 0  )
-				printf("Fragmented packet: last segement: ip_off: %u, frag_offset: %u\n", ip_off, frag_offset);
+				printf("Fragmented packet: last segement: ip_off: %u, frag_offset: %u\n",
+					ip_off, frag_offset);
 #endif
 			// fragmented packet
-			defragmented = IPFrag_tree_Update(ip->ip_src.s_addr, ip->ip_dst.s_addr, ip_id, &payload_len, ip_off, payload);
+			defragmented = IPFrag_tree_Update(hdr->ts.tv_sec, ip->ip_src.s_addr, ip->ip_dst.s_addr,
+				ip_id, &payload_len, ip_off, payload);
 			if ( defragmented == NULL ) {
 				// not yet complete
-				dbg_printf("Fragmentation not yet completed\n");
-				return;
+				dbg_printf("Fragmentation not yet completed. Size %u bytes\n", payload_len);
+				Free_Node(Node);
+				goto END_FUNC;
 			}
-			dbg_printf("Fragmentation assembled\n");
+			dbg_printf("Fragmentation complete\n");
 			// packet defragmented - set payload to defragmented data
 			payload = defragmented;
-		} 
+		}
+		bytes 		= payload_len;
 
 		Node->src_addr.v6[0] = 0;
 		Node->src_addr.v6[1] = 0;
@@ -602,10 +593,10 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		Node->dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
 		Node->version = AF_INET;
 	} else {
-		LogError("ProcessPacket() Unsupprted protocol version: %i", version);
+		LogInfo("ProcessPacket() Unsupprted protocol version: %i", version);
 		pcap_dev->proc_stat.unknown++;
 		Free_Node(Node);
-		return;
+		goto END_FUNC;
 	}
 
 	Node->packets = 1;
@@ -619,31 +610,36 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			struct udphdr *udp = (struct udphdr *)payload;
 			uint16_t UDPlen = ntohs(udp->uh_ulen);
 			if ( UDPlen < 8 ) {
-				LogError("UDP payload legth error: %u bytes < 8\n",
-					UDPlen);
+				LogInfo("UDP payload length error: %u bytes < 8, SRC %s, DST %s",
+					UDPlen, inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
+					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
+
 				Free_Node(Node);
 				break;
 			}
 			uint32_t size_udp_payload = ntohs(udp->uh_ulen) - 8;
 
-			if ( (bytes == payload_len ) && (payload_len - sizeof(struct udphdr)) != size_udp_payload ) {
-				LogError("UDP payload legth error: Expected %u, have %u bytes\n",
-					size_udp_payload, (payload_len - (unsigned)sizeof(struct udphdr)));
+			if ( (bytes == payload_len ) && (payload_len - sizeof(struct udphdr)) < size_udp_payload ) {
+				LogInfo("UDP payload length error: Expected %u, have %u bytes, SRC %s, DST %s",
+					size_udp_payload, (payload_len - (unsigned)sizeof(struct udphdr)),
+					inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
+					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
 				Free_Node(Node);
 				break;
 			}
 			payload = payload + sizeof(struct udphdr);
 			payload_len -= sizeof(struct udphdr);
-			dbg_printf("UDP: size: %u, SRC: %i, DST: %i\n", 
+			dbg_printf("UDP: size: %u, SRC: %i, DST: %i\n",
 				size_udp_payload, ntohs(udp->uh_sport), ntohs(udp->uh_dport));
 
+			Node->bytes = payload_len;
 			Node->flags = 0;
 			Node->src_port = ntohs(udp->uh_sport);
 			Node->dst_port = ntohs(udp->uh_dport);
 
 			if ( hdr->caplen == hdr->len ) {
 				// process payload of full packets
-				if ( (bytes == payload_len) && (Node->src_port == 53 || Node->dst_port == 53) ) 
+				if ( (bytes == payload_len) && (Node->src_port == 53 || Node->dst_port == 53) )
  					content_decode_dns(Node, payload, payload_len);
 			}
 			Push_Node(NodeList, Node);
@@ -654,7 +650,12 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			size_tcp = tcp->th_off << 2;
 
 			if ( payload_len < size_tcp ) {
-				LogError("TCP header length error: len: %u < size TCP header: %u", payload_len, size_tcp);	
+				LogInfo("TCP header length error: len: %u < size TCP header: %u, SRC %s, DST %s",
+					payload_len, size_tcp,
+					inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
+					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
+                pcap_dev->proc_stat.short_snap++;
+
 				pcap_dev->proc_stat.short_snap++;
 				Free_Node(Node);
 				break;
@@ -663,7 +664,7 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			payload = payload + size_tcp;
 			payload_len -= size_tcp;
 			dbg_printf("Size TCP header: %u, size TCP payload: %u ", size_tcp, payload_len);
-			dbg_printf("src %i, DST %i, flags %i : ", 
+			dbg_printf("src %i, DST %i, flags %i : ",
 				ntohs(tcp->th_sport), ntohs(tcp->th_dport), tcp->th_flags);
 
 #ifdef DEVEL
@@ -686,28 +687,28 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			struct icmp *icmp = (struct icmp *)payload;
 
 			Node->dst_port = (icmp->icmp_type << 8 ) + icmp->icmp_code;
-			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n", version, ip->ip_p, icmp->icmp_type, icmp->icmp_code);
+			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n",
+				version, ip->ip_p, icmp->icmp_type, icmp->icmp_code);
+			Node->bytes -= sizeof(struct udphdr);
 			Push_Node(NodeList, Node);
 			} break;
 		case IPPROTO_ICMPV6: {
 			struct icmp6_hdr *icmp6 = (struct icmp6_hdr *)payload;
 
 			Node->dst_port = (icmp6->icmp6_type << 8 ) + icmp6->icmp6_code;
-			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n", version, ip->ip_p, icmp6->icmp6_type, icmp6->icmp6_code);
+			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n",
+				version, ip->ip_p, icmp6->icmp6_type, icmp6->icmp6_code);
 			Push_Node(NodeList, Node);
 			} break;
 		case IPPROTO_IPV6: {
 			uint32_t size_inner_ip = sizeof(struct ip6_hdr);
 
 			if ( payload_len < size_inner_ip ) {
-				LogError("IPIPv6 tunnel header length error: len: %u < size inner IP: %u", payload_len, size_inner_ip);	
+				LogInfo("IPIPv6 tunnel header length error: len: %u < size inner IP: %u",
+					payload_len, size_inner_ip);	
 				pcap_dev->proc_stat.short_snap++;
-				if ( defragmented ) {
-					free(defragmented);
-					defragmented = NULL;
-				}
 				Free_Node(Node);
-				return;
+				goto END_FUNC;
 			}
 			offset   = 0;
 			data 	 = payload;
@@ -728,7 +729,8 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint32_t size_inner_ip = (inner_ip->ip_hl << 2);
 
 			if ( payload_len < size_inner_ip ) {
-				LogError("IPIP tunnel header length error: len: %u < size inner IP: %u", payload_len, size_inner_ip);	
+				LogInfo("IPIP tunnel header length error: len: %u < size inner IP: %u",
+					payload_len, size_inner_ip);	
 				pcap_dev->proc_stat.short_snap++;
 				Free_Node(Node);
 				break;
@@ -753,7 +755,8 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint32_t gre_hdr_size = sizeof(gre_hdr_t); // offset points to end of inner IP
 
 			if ( payload_len < gre_hdr_size ) {
-				LogError("GRE tunnel header length error: len: %u < size GRE hdr: %u", payload_len, gre_hdr_size);	
+				LogError("GRE tunnel header length error: len: %u < size GRE hdr: %u",
+					payload_len, gre_hdr_size);	
 				pcap_dev->proc_stat.short_snap++;
 				Free_Node(Node);
 				break;
@@ -774,13 +777,14 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			goto REDO_LINK;
 
 			} break;
-		default: 
+		default:
 			// not handled protocol - simply save node
 			Push_Node(NodeList, Node);
 			pcap_dev->proc_stat.unknown++;
 			break;
 	}
 
+	END_FUNC:
 	if ( defragmented ) {
 		free(defragmented);
 		defragmented = NULL;
@@ -789,6 +793,3 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 
 
 } // End of ProcessPacket
-
-
-
