@@ -1,7 +1,6 @@
 /*
- *  nfreplay :  Reads netflow data from files saved by nfcapd
- *  			and sends them to another host.
- *
+ *  Copyright (c) 2014, Peter Haag
+ *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -9,13 +8,13 @@
  *  modification, are permitted provided that the following conditions are met:
  *  
  *   * Redistributions of source code must retain the above copyright notice, 
- *	 this list of conditions and the following disclaimer.
+ *	   this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above copyright notice, 
- *	 this list of conditions and the following disclaimer in the documentation 
- *	 and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
- *	 used to endorse or promote products derived from this software without 
- *	 specific prior written permission.
+ *	   this list of conditions and the following disclaimer in the documentation 
+ *	   and/or other materials provided with the distribution.
+ *   * Neither the name of the author nor the names of its contributors may be 
+ *	   used to endorse or promote products derived from this software without 
+ *	   specific prior written permission.
  *  
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
@@ -29,11 +28,11 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
+ *  $Author: peter $
  *
- *  $Id: nftrack.c 8 2009-05-07 08:13:13Z haag $
+ *  $Id: nftrack.c 224 2014-02-16 12:59:29Z peter $
  *
- *  $LastChangedRevision: 8 $
+ *  $LastChangedRevision: 224 $
  *	
  */
 
@@ -46,6 +45,8 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/param.h>
+#include <signal.h>
 #include <fcntl.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -81,7 +82,7 @@ FilterEngine_data_t	*Engine;
 int 		byte_mode, packet_mode;
 uint32_t	byte_limit, packet_limit;	// needed for linking purpose only
 
-extension_map_list_t extension_map_list;
+extension_map_list_t *extension_map_list;
 
 /* Local Variables */
 static const char *nfdump_version = VERSION;
@@ -89,6 +90,8 @@ static const char *nfdump_version = VERSION;
 /* Function Prototypes */
 static void usage(char *name);
 
+static int CheckRunningOnce(char *pidfile);
+
 static data_row *process(char *filter);
 
 /* Functions */
@@ -112,6 +115,57 @@ static void usage(char *name) {
 					, name);
 } /* usage */
 
+static int CheckRunningOnce(char *pidfile) {
+int pidf;
+pid_t pid;
+char pidstr[32];
+
+	pidf = open(pidfile, O_RDONLY, 0);
+	if ( pidf > 0 ) {
+		// pid file exists
+		char s[32];
+		ssize_t len;
+		len = read(pidf, (void *)s, 31);
+		close(pidf);
+		s[31] = '\0';
+		if ( len < 0 ) {
+			LogError("read() error existing pid file: %s\n", strerror(errno));
+			return 0;
+		} else {
+			unsigned long pid = atol(s);
+			if ( pid == 0 ) {
+				// garbage - use this file
+				unlink(pidfile);
+			} else {
+				if ( kill(pid, 0) == 0 ) {
+					// process exists
+					LogError("An nftrack process with pid %lu is already running!\n", pid);
+					return 0;
+				} else {
+					// no such process - use this file
+					LogError("The nftrack process with pid %lu died unexpectedly!\n", pid);
+					unlink(pidfile);
+				}
+			}
+		}
+	} 
+
+	pid = getpid();
+	pidf  = open(pidfile, O_RDWR|O_TRUNC|O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+	if ( pidf == -1 ) {
+		LogError("Error opening nftrack pid file: '%s' %s", pidfile, strerror(errno));
+		return 0;
+	}
+	snprintf(pidstr,31,"%lu\n", (unsigned long)pid);
+	if ( write(pidf, pidstr, strlen(pidstr)) <= 0 ) {
+		LogError("Error write nftrack pid file: '%s' %s", pidfile, strerror(errno));
+	}
+	close(pidf);
+
+	return 1;
+
+} // End of CheckRunningOnce
+
 static data_row *process(char *filter) {
 master_record_t		master_record;
 common_record_t		*flow_record;
@@ -125,6 +179,10 @@ uint64_t total_bytes;
 		LogError("GetNextFile() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		return NULL;
 	}
+	if ( nffile == EMPTY_LIST ) {
+		LogError("Empty file list. No files to process\n");
+		return NULL;
+	}
 
 	port_table    = (data_row *)calloc(65536, sizeof(data_row));
     if ( !port_table) {
@@ -138,6 +196,7 @@ uint64_t total_bytes;
 	// is expanded into this record
 	Engine->nfrecord = (uint64_t *)&master_record;
 
+	total_bytes = 0;
 	done	 	= 0;
 	while ( !done ) {
 
@@ -148,9 +207,9 @@ uint64_t total_bytes;
             case NF_CORRUPT:
             case NF_ERROR:
                 if ( ret == NF_CORRUPT ) 
-                    fprintf(stderr, "Skip corrupt data file '%s'\n",GetCurrentFilename());
+                    LogError("Skip corrupt data file '%s'\n",GetCurrentFilename());
                 else 
-                    fprintf(stderr, "Read error in file '%s': %s\n",GetCurrentFilename(), strerror(errno) );
+                    LogError("Read error in file '%s': %s\n",GetCurrentFilename(), strerror(errno) );
                 // fall through - get next file in chain
             case NF_EOF: {
 				nffile_t *next = GetNextFile(nffile, 0, 0);
@@ -176,7 +235,7 @@ uint64_t total_bytes;
 		}
 
 		if ( nffile->block_header->id != DATA_BLOCK_TYPE_2 ) {
-			fprintf(stderr, "Can't process block type %u\n", nffile->block_header->id);
+			LogError("Can't process block type %u\n", nffile->block_header->id);
 			continue;
 		}
 
@@ -185,43 +244,51 @@ uint64_t total_bytes;
 		for ( i=0; i < nffile->block_header->NumRecords; i++ ) {
 			int			ret;
 
-            if ( flow_record->type == CommonRecordType ) {
-                if ( extension_map_list.slot[flow_record->ext_map] == NULL ) {
-                    LogError("Corrupt data file! No such extension map id: %u. Skip record", flow_record->ext_map );
-                } else {
-                    ExpandRecord_v2( flow_record, extension_map_list.slot[flow_record->ext_map], &master_record);
+			switch ( flow_record->type ) {
+				case CommonRecordV0Type:
+				case CommonRecordType: {
+                	if ( extension_map_list->slot[flow_record->ext_map] == NULL ) {
+                    	LogError("Corrupt data file! No such extension map id: %u. Skip record", flow_record->ext_map );
+                	} else {
+                    	ExpandRecord_v2( flow_record, extension_map_list->slot[flow_record->ext_map], NULL, &master_record);
             
-   					ret = (*Engine->FilterEngine)(Engine);
+   						ret = (*Engine->FilterEngine)(Engine);
 
-					if ( ret == 0 ) { // record failed to pass the filter
-						// increment pointer by number of bytes for netflow record
-						flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
-						// go to next record
-						continue;
-					}
+						if ( ret == 0 ) { // record failed to pass the filter
+							// increment pointer by number of bytes for netflow record
+							flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
+							// go to next record
+							continue;
+						}
 
 
-					// Add to stat record
-					if ( master_record.prot == 6 ) {
-						port_table[master_record.dstport].proto[tcp].type[flows]++;
-						port_table[master_record.dstport].proto[tcp].type[packets]	+= master_record.dPkts;
-						port_table[master_record.dstport].proto[tcp].type[bytes]	+= master_record.dOctets;
-					} else if ( master_record.prot == 17 ) {
-						port_table[master_record.dstport].proto[udp].type[flows]++;
-						port_table[master_record.dstport].proto[udp].type[packets]	+= master_record.dPkts;
-						port_table[master_record.dstport].proto[udp].type[bytes]	+= master_record.dOctets;
-					}
-             	}
+						// Add to stat record
+						if ( master_record.prot == 6 ) {
+							port_table[master_record.dstport].proto[tcp].type[flows]++;
+							port_table[master_record.dstport].proto[tcp].type[packets]	+= master_record.dPkts;
+							port_table[master_record.dstport].proto[tcp].type[bytes]	+= master_record.dOctets;
+						} else if ( master_record.prot == 17 ) {
+							port_table[master_record.dstport].proto[udp].type[flows]++;
+							port_table[master_record.dstport].proto[udp].type[packets]	+= master_record.dPkts;
+							port_table[master_record.dstport].proto[udp].type[bytes]	+= master_record.dOctets;
+						}
+             		}
+				} break;
+				case ExtensionMapType: {
+                	extension_map_t *map = (extension_map_t *)flow_record;
 
-            } else if ( flow_record->type == ExtensionMapType ) {
-                extension_map_t *map = (extension_map_t *)flow_record;
-
-                if ( Insert_Extension_Map(&extension_map_list, map) ) {
-                     // flush new map
-                } // else map already known and flushed
-
-            } else {
-                fprintf(stderr, "Skip unknown record type %i\n", flow_record->type);
+                	if ( Insert_Extension_Map(extension_map_list, map) ) {
+                     		// flush new map
+                	} // else map already known and flushed
+				} break;
+				case ExporterInfoRecordType:
+				case ExporterStatRecordType:
+				case SamplerInfoRecordype:
+						// Silently skip exporter records
+					break;
+				default: {
+					LogError("Skip unknown record type %i\n", flow_record->type);
+				}
             }
 
 			// Advance pointer by number of bytes for netflow record
@@ -232,6 +299,8 @@ uint64_t total_bytes;
 	CloseFile(nffile);
 	DisposeFile(nffile);
 
+	PackExtensionMapList(extension_map_list);
+
 	return port_table;
 
 } // End of process
@@ -239,10 +308,11 @@ uint64_t total_bytes;
 
 int main( int argc, char **argv ) {
 struct stat stat_buff;
-char c, *wfile, *rfile, *Rfile, *Mdirs, *ffile, *filter, *timeslot, *DBdir;
+char *wfile, *rfile, *Rfile, *Mdirs, *ffile, *filter, *timeslot, *DBdir;
 char datestr[64];
-int ffd, ret, DBinit, AddDB, GenStat, AvStat, output_mode;
-unsigned int lastupdate, topN;
+char pidfile[MAXPATHLEN];
+int c, ffd, ret, DBinit, AddDB, GenStat, AvStat, output_mode, topN;
+unsigned int lastupdate;
 data_row *port_table;
 time_t	when;
 struct tm * t1;
@@ -332,22 +402,22 @@ struct tm * t1;
 
 	if ( !filter && ffile ) {
 		if ( stat(ffile, &stat_buff) ) {
-			perror("Can't stat file");
+			LogError("stat() filter file: '%s' %s", ffile, strerror(errno));
 			exit(255);
 		}
 		filter = (char *)malloc(stat_buff.st_size);
 		if ( !filter ) {
-			perror("Memory error");
+			LogError("malloc() allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 			exit(255);
 		}
 		ffd = open(ffile, O_RDONLY);
 		if ( ffd < 0 ) {
-			perror("Can't open file");
+			LogError("open() filter file: '%s' %s", ffile, strerror(errno));
 			exit(255);
 		}
 		ret = read(ffd, (void *)filter, stat_buff.st_size);
 		if ( ret < 0   ) {
-			perror("Error reading file");
+			LogError("read() filter file: '%s' %s", ffile, strerror(errno));
 			close(ffd);
 			exit(255);
 		}
@@ -355,38 +425,55 @@ struct tm * t1;
 	}
 
 	if ( !DBdir ) {
-		fprintf(stderr, "DB directory required\n");
+		LogError("DB directory required\n");
 		exit(255);
 	}
 
 	InitStat(DBdir);
 
+	// check if pid file exists and if so, if a process with registered pid is running
+	snprintf(pidfile, MAXPATHLEN-1, "%s/nftrack.pid", DBdir);
+	pidfile[MAXPATHLEN-1]= '\0';
+	if ( !CheckRunningOnce(pidfile) ) {
+		LogError("Run once check failed.\n");
+		exit(255);
+	}
+
 	if ( !filter )
 		filter = "any";
 
 	Engine = CompileFilter(filter);
-	if ( !Engine ) 
+	if ( !Engine ) {
+		unlink(pidfile);
 		exit(254);
-	
+	}
+
 	if ( DBinit ) {
 		when = time(NULL);
 		when -= ((when % 300) + 300);
 		InitStatFile();
 		if ( !CreateRRDBs(DBdir, when) ) {
-			fprintf(stderr, "Init DBs failed\n");
+			LogError("Init DBs failed\n");
+			unlink(pidfile);
 			exit(255);
 		}
-		fprintf(stderr, "Port DBs initialized.\n");
+		LogInfo("Port DBs initialized.\n");
+		unlink(pidfile);
 		exit(0);
 	}
 
+	extension_map_list = InitExtensionMaps(NEEDS_EXTENSION_LIST);
+
 	if ( lastupdate ) {
 		when = RRD_LastUpdate(DBdir);
-		if ( !when )
+		if ( !when ) {
+			unlink(pidfile);
 			exit(255);
+		}
 		t1 = localtime(&when);
 		strftime(datestr, 63, "%b %d %Y %T", t1);
-		printf("Last Update: %i, %s\n", (int)when, datestr);
+		LogInfo("Last Update: %i, %s\n", (int)when, datestr);
+		unlink(pidfile);
 		exit(0);
 	}
 
@@ -396,11 +483,13 @@ struct tm * t1;
 		port_table = process(filter);
 //		Lister(port_table);
 		if ( !port_table ) {
+			unlink(pidfile);
 			exit(255);
 		}
 		if ( AddDB ) {
 			if ( !timeslot ) {
-				fprintf(stderr, "Timeslot required!\n");
+				LogError("Timeslot required!\n");
+				unlink(pidfile);
 				exit(255);
 			}
 			UpdateStat(port_table, ISO2UNIX(timeslot));
@@ -411,7 +500,8 @@ struct tm * t1;
 	if ( AvStat ) {
 		port_table = GetStat();
 		if ( !port_table ) {
-			fprintf(stderr, "Unable to get port table!\n");
+			LogError("Unable to get port table!\n");
+			unlink(pidfile);
 			exit(255);
 		}
 		// DoStat
@@ -424,13 +514,15 @@ struct tm * t1;
 		when = ISO2UNIX(timeslot);
 		if ( !port_table ) {
 			if ( !timeslot ) {
-				fprintf(stderr, "Timeslot required!\n");
+				LogError("Timeslot required!\n");
+				unlink(pidfile);
 				exit(255);
 			}
 			port_table = RRD_GetDataRow(DBdir, when);
 		}
 		if ( !port_table ) {
-			fprintf(stderr, "Unable to get port table!\n");
+			LogError("Unable to get port table!\n");
+			unlink(pidfile);
 			exit(255);
 		}
 		// DoStat
@@ -438,6 +530,7 @@ struct tm * t1;
 	}
 
 	CloseStat();
+	unlink(pidfile);
 
 	return 0;
 }
